from __future__ import print_function

import os
import sys
from code import InteractiveConsole
from atexit import register

from canari.mode import CanariMode, set_canari_mode
from canari.pkgutils.transform import TransformDistribution
from canari.commands.common import canari_main
from canari.question import parse_bool
from canari.utils.fs import PushDir
from canari.commands.framework import SubCommand, Argument
from canari.config import load_config
from canari.maltego.utils import highlight
from canari.maltego.runner import scriptable_transform_runner
import canari

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.5'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def sudo():
    print(highlight("Need to be root to run this transform... sudo'ing...", 'green', True), file=sys.stderr)
    sys.argv.insert(0, 'sudo')
    os.execvp(sys.argv[0], sys.argv)


class ShellCommand(object):

    def __init__(self, transform, config):
        self.transform = transform
        self.config = config

    def __call__(self, value, *args, **kwargs):
        if os.name == 'posix' and self.transform.superuser and os.geteuid():
            sudo()
        if args and isinstance(args[0], dict):
            kwargs.update(args[0])
            args = args[1:]
        return scriptable_transform_runner(self.transform, value, kwargs, list(args), self.config)


class MtgConsole(InteractiveConsole):

    def __init__(self, transform_classes, auto_sudo=False):
        locals_ = {}
        asked = False
        config = load_config()
        for transform_class in transform_classes:
            transform = transform_class()
            locals_['do' + transform.name.split('.')[-1]] = ShellCommand(transform, config)
            if not asked and transform.superuser and os.name == 'posix' and os.geteuid():
                if not auto_sudo and parse_bool("A transform requiring 'root' access was detected."
                                                " Would you like to run this shell as 'root'?", False):
                    sudo()
                asked = True

        InteractiveConsole.__init__(self, locals=locals_)
        MtgConsole.init_history(os.path.expanduser('~/.mtgsh_history'))

    @staticmethod
    def init_history(history_file):
        try:
            import readline
            readline.parse_and_bind('tab: complete')
            if hasattr(readline, "read_history_file"):
                try:
                    readline.read_history_file(history_file)
                except IOError:
                    pass
                register(lambda h: readline.write_history_file(h), history_file)
        except ImportError:
            pass


@SubCommand(
    canari_main,
    help='Creates a Canari debug shell for the specified transform package.',
    description='Creates a Canari debug shell for the specified transform package.'
)
@Argument(
    'package',
    metavar='<package name>',
    help='The name of the canari package you wish to load local transform from for the Canari shell session.'
)
@Argument(
    '-w',
    '--working-dir',
    metavar='[working dir]',
    default=None,
    help="the path that will be used as the working directory for "
         "the transforms being executed in the shell (default: ~/.canari/)"
)
@Argument(
    '-s',
    '--sudo',
    action='store_true',
    default=False,
    help='Instructs the shell to automatically elevate privileges to root if necessary.'
)
def shell(opts):

    set_canari_mode(CanariMode.LocalShellDebug)

    if not opts.package.endswith('transforms'):
        opts.package = '%s.transforms' % opts.package

    try:
        transform_package = TransformDistribution(opts.package)
        with PushDir(opts.working_dir or transform_package.default_prefix):
            mtg_console = MtgConsole(transform_package.transforms, auto_sudo=opts.sudo)
            mtg_console.interact(highlight('Welcome to Canari %s.' % canari.__version__, 'green', True))
    except ValueError as e:
        print(str(e), file=sys.stderr)
        exit(-1)



from canari.commands.common import canari_main
from canari.commands.framework import SubCommand


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help='Show banner of Canari framework that is currently active.',
    description='Show banner of Canari framework that is currently active.'
)
def banner(args):
    print(r"""
    Your running ...
                                 _     ___ 
         _______ ____  ___ _____(_)   /_  /
        / __/ _ `/ _ \/ _ `/ __/ /   /_  /
        \__/\_,_/_//_/\_,_/_/ /_/   /___/

                                            ... http://github.com/redcanari/canari3
    """)


from __future__ import print_function

import sys
import os
from mimetypes import guess_type
import shutil

import boto3
from hashlib import md5

from six import b

if sys.version_info[0] > 2:
    from queue import Queue
else:
    from Queue import Queue

from mrbob.configurator import Configurator

from canari.commands.common import canari_main
from canari.commands.framework import SubCommand, Argument
from canari.pkgutils.transform import TransformDistribution
from canari.resource import image_resources
from canari.project import CanariProject
from canari.utils.fs import PushDir

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


S3_CDN_BUCKET_POLICY = """{
  "Version":"2012-10-17",
  "Statement":[{
	"Sid":"PublicReadGetObject",
        "Effect":"Allow",
	  "Principal": "*",
      "Action":["s3:GetObject"],
      "Resource":["arn:aws:s3:::%s/*"
      ]
    }
  ]
}"""

# Queue for blocking until we get a result from hook to setup(). Ugly but effective :P
q = Queue()


def setup_hook(**kwargs):
    print("Gathering package requirements for %s..." % kwargs['name'], file=sys.stderr)
    q.put(([r for r in kwargs.get('install_requires', []) if not r.startswith('canari')]))


def hook_setup():
    import setuptools
    setuptools.setup = setup_hook


def get_dependencies(project):
    with PushDir(project.root_dir):
        hook_setup()
        import setup
    return set(q.get())


def upload_images(project, opts):
    bucket = opts.bucket or md5(b('canari-%s-transform-images' % project.name)).hexdigest()
    region = opts.region_name or boto3.session.Session().region_name
    print('Uploading image resources to AWS S3 bucket %r' % bucket, file=sys.stderr)

    s3 = boto3.client(
        's3',
        # Hard coded strings as credentials, not recommended.
        aws_access_key_id=opts.aws_access_key_id,
        aws_secret_access_key=opts.aws_secret_access_key,
        region_name=region
    )

    if bucket not in [bucket['Name'].lower() for bucket in s3.list_buckets()['Buckets']]:
        print('Creating S3 bucket %r because it does not exist' % bucket)
        s3.create_bucket(Bucket=bucket, CreateBucketConfiguration={'LocationConstraint': region})

    with PushDir(project.src_dir):
        for i in image_resources(project.name):
            img_name = os.path.basename(i)
            dst_path = 'static/%s' % md5(b(img_name)).hexdigest()
            print('Uploading %r to %r...' % (i, dst_path))
            s3.upload_file(i, bucket, dst_path, ExtraArgs={'ContentType': guess_type(i)[0]})

    print('Creating static website configuration for bucket %r...' % bucket)
    s3.put_bucket_website(Bucket=bucket, WebsiteConfiguration={'IndexDocument': {'Suffix': 'index.html'}})

    print('Adjusting bucket policy for public GetObject access...')
    s3.put_bucket_policy(Bucket=bucket, Policy=S3_CDN_BUCKET_POLICY % bucket)

    base_url = 'http://%s.s3-website.%s.amazonaws.com/static/' % (bucket, region)
    print('Static images are now hosted at %r!' % base_url)

    return base_url



@SubCommand(
    canari_main,
    help="Adds AWS Lambda capability.",
    description="Creates an AWS Chalice project for deployment to lambda."
)
@Argument(
    '-b',
    '--bucket',
    metavar='<S3 bucket name>',
    help="The name of the bucket to store image and other binary resources for transforms in."
)
@Argument(
    '-r',
    '--region-name',
    default=None,
    metavar='<AWS region>',
    help="The region to store the S3 objects in"
)
@Argument(
    '-k',
    '--aws-access-key-id',
    default=None,
    metavar='<AWS access key ID>',
    help="AWS access key ID"
)
@Argument(
    '-s',
    '--aws-secret-access-key',
    metavar='<AWS secret access key>',
    help="AWS secret access key"
)
def create_aws_lambda(opts):
    try:
        project = CanariProject()
        target = os.path.join(project.root_dir, 'aws')

        cdn_base_url = upload_images(project, opts)

        with PushDir(project.src_dir):
            transform_package = TransformDistribution(project.name)
            variables = {
                'project.name': project.name,
                'project.requirements': get_dependencies(project),
                'package.transforms': [t() for t in transform_package.transforms]
            }

            configurator = Configurator(
                'canari.resources.templates:create_aws_lambda',
                target,
                {'non_interactive': True},
                variables=variables
            )

            configurator.ask_questions()

            print('Generating Chalice project for %r in %r...' % (project.name, target), file=sys.stderr)
            configurator.render()

            transform_package.configure(os.path.join(target, 'chalicelib'), remote=True, additional_options={
                'cdn': cdn_base_url
            })

            if os.path.lexists('../aws/foo'):
                shutil.rmtree('../aws/foo')

            print("Copying %r project tree to the 'aws' directory." % project.name)
            shutil.copytree('foo', '../aws/foo',
                            ignore=shutil.ignore_patterns("__pycache__", "*.pyc", ".*", "*.gif", "*.jpg", "*.jpeg", "*.png"))

            print("To deploy type 'chalice deploy' in the %r directory" % target)
            print('done!', file=sys.stderr)
    except ValueError as e:
        print(e, file=sys.stderr)
        exit(-1)



from __future__ import print_function

import sys
from csv import writer
import os
from xml.etree.cElementTree import XML
from zipfile import ZipFile

from canari.commands.framework import SubCommand, Argument
from canari.commands.common import canari_main, to_utf8


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.5'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help='Convert Maltego graph files (*.mtgx) to comma-separated values (CSV) file.',
    description='Convert Maltego graph files (*.mtgx) to comma-separated values (CSV) file.'
)
@Argument(
    'graph',
    metavar='<graph>',
    help='The name of the graph file you wish to convert to CSV.',
)
def mtgx2csv(opts):

    zipfile = ZipFile(opts.graph)
    graphs = filter(lambda x: x.endswith('.graphml'), zipfile.namelist())

    for f in graphs:
        filename = '%s_%s' % (opts.graph.replace('.', '_', 1), os.path.basename(f).replace('.graphml', '.csv', 1))
        print('Writing data from %s/%s to %s...' % (opts.graph, f, filename), file=sys.stderr)
        with open(filename, 'w') as csvfile:
            csv = writer(csvfile)
            xml = XML(zipfile.open(f).read())
            links = {}
            for edge in xml.findall('{http://graphml.graphdrawing.org/xmlns}graph/'
                                    '{http://graphml.graphdrawing.org/xmlns}edge'):
                src = edge.get('source')
                dst = edge.get('target')
                if src not in links:
                    links[src] = dict(in_=0, out=0)
                if dst not in links:
                    links[dst] = dict(in_=0, out=0)
                links[src]['out'] += 1
                links[dst]['in_'] += 1

            for node in xml.findall('{http://graphml.graphdrawing.org/xmlns}graph/'
                                    '{http://graphml.graphdrawing.org/xmlns}node'):

                node_id = node.get('id')
                node = node.find('{http://graphml.graphdrawing.org/xmlns}data/'
                                 '{http://maltego.paterva.com/xml/mtgx}MaltegoEntity')

                row = [to_utf8(('Entity Type=%s' % node.get('type')).strip())]
                for prop in node.findall('{http://maltego.paterva.com/xml/mtgx}Properties/'
                                         '{http://maltego.paterva.com/xml/mtgx}Property'):
                    value = prop.find('{http://maltego.paterva.com/xml/mtgx}Value').text or ''
                    row.append(to_utf8(('%s=%s' % (prop.get('displayName'), value)).strip()))
                row.append('Incoming Links=%s' % links.get(node_id, {}).get('in_', 0))
                row.append('Outgoing Links=%s' % links.get(node_id, {}).get('out', 0))
                csv.writerow(row)


#!/usr/bin/env python

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = ['Nadeem Douba']

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'maltego',
    'transform'
]


import sys

from past.builtins import basestring

from canari.utils.common import find_pysudo

if sys.version_info[0] > 2:
    from http.client import HTTPSConnection, HTTPConnection
else:
    # noinspection PyUnresolvedReferences
    from httplib import HTTPSConnection, HTTPConnection

import subprocess
import os
import sys
import traceback
from collections import defaultdict
from canari.mode import is_debug_exec_mode
from importlib import import_module

import re
from xml.etree.cElementTree import fromstring

from safedexml import Model

from canari.config import load_config
from canari.maltego.message import MaltegoTransformResponseMessage, UIMessage, MaltegoTransformRequestMessage, Field, \
    MaltegoException, EntityTypeFactory, Entity, MaltegoMessage, Limits
from canari.maltego.utils import message, on_terminate, to_entity, croak, highlight

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = []


def sudo(args):
    p = subprocess.Popen([find_pysudo()] + args)
    p.communicate()
    return p.returncode


def load_object(classpath):
    package, cls = re.search(r'^(.*)\.([^.]+)$', classpath).groups()
    module = import_module(package)
    return module.__dict__[cls]


def remote_canari_transform_runner(host, base_path, transform, entities, parameters, limits, is_ssl=False):
    c = HTTPSConnection(host) if is_ssl else HTTPConnection(host)

    m = MaltegoTransformRequestMessage()

    for e in entities:
        m += e

    for p in parameters:
        m += p

    m += limits

    message = MaltegoMessage(message=m).render()
    path = re.sub(r'/+', '/', '/'.join([base_path, transform]))

    if is_debug_exec_mode():
        sys.stderr.write("Sending following message to {}{}:\n{}\n\n".format(host, path, message))

    c.request('POST', path, message, headers={'Content-Type': 'application/xml'})

    return c.getresponse()


def local_transform_runner(transform, value, fields, params, config, message_writer=message):
    """
    Internal API: The local transform runner is responsible for executing the local transform.

    Parameters:

    transform      - The name or module of the transform to execute (i.e sploitego.transforms.whatismyip).
    value          - The input entity value.
    fields         - A dict of the field names and their respective values.
    params         - The extra parameters passed into the transform via the command line.
    config         - The Canari configuration object.
    message_writer - The message writing function used to write the MaltegoTransformResponseMessage to stdout. This is
                     can either be the console_message or message functions. Alternatively, the message_writer function
                     can be any callable object that accepts the MaltegoTransformResponseMessage as the first parameter
                     and writes the output to a destination of your choosing.

    This helper function is only used by the run-transform, debug-transform, and dispatcher commands.
    """

    try:
        transform = load_object(transform)()

        if os.name == 'posix' and transform.superuser and os.geteuid():
            rc = sudo(sys.argv)
            if rc == 1:
                message_writer(MaltegoTransformResponseMessage() + UIMessage('User cancelled transform.'))
            elif rc == 2:
                message_writer(MaltegoTransformResponseMessage() + UIMessage('Too many incorrect password attempts.'))
            elif rc:
                message_writer(MaltegoTransformResponseMessage() + UIMessage('Unknown error occurred.'))
            exit(rc)

        on_terminate(transform.on_terminate)

        request = MaltegoTransformRequestMessage(
            parameters={'canari.local.arguments': Field(name='canari.local.arguments', value=params)}
        )

        request._entities = [to_entity(transform.input_type, value, fields)]
        request.limits = Limits(soft=10000)

        msg = transform.do_transform(
            request,
            MaltegoTransformResponseMessage(),
            config
        )
        if isinstance(msg, MaltegoTransformResponseMessage):
            message_writer(msg)
        elif isinstance(msg, basestring):
            raise MaltegoException(msg)
        else:
            raise MaltegoException('Could not resolve message type returned by transform.')
    except MaltegoException as me:
        croak(me, message_writer)
    except KeyboardInterrupt:
        # Ensure that the keyboard interrupt handler does not execute twice if a transform is sudo'd
        if (transform.superuser and not os.geteuid()) or (not transform.superuser and os.geteuid()):
            transform.on_terminate()
    except Exception:
        croak(traceback.format_exc(), message_writer)


class Response(object):
    def __init__(self, maltego_response):
        self._response = maltego_response
        self._entities = [EntityTypeFactory.create(e.type)(e) for e in maltego_response.entities]
        self._messages = defaultdict(list)
        for m in maltego_response.messages:
            self._messages[m.type].append(m.message)

    def toXML(self):
        return self._response.render(fragment=True)

    @property
    def entities(self):
        return self._entities

    @property
    def messages(self):
        return self._messages


scriptable_api_initialized = False


def scriptable_transform_runner(transform, value, fields, params, config):
    global scriptable_api_initialized
    if not scriptable_api_initialized:
        scriptable_api_initialized = True

        def run_transform(self, transform, params=None, config=None):
            if isinstance(transform, basestring):
                transform = load_object(transform)()
            return scriptable_transform_runner(
                transform,
                self.value,
                self.fields,
                params or [],
                config or load_config()
            )

        Entity.run_transform = run_transform

    request = MaltegoTransformRequestMessage(
        parameters={'canari.local.arguments': Field(name='canari.local.arguments', value=params)}
    )

    request._entities = [to_entity(transform.input_type, value, fields)]
    request.limits = Limits(soft=10000)

    msg = transform.do_transform(
        request,
        MaltegoTransformResponseMessage(),
        config
    )
    if isinstance(msg, MaltegoTransformResponseMessage):
        return Response(msg)
    elif isinstance(msg, basestring):
        raise MaltegoException(msg)
    else:
        raise MaltegoException('Could not resolve message type returned by transform.')


def console_writer(msg, tab=-1):
    """
    Internal API: Returns a prettified tree-based output of an XML message for debugging purposes. This helper function
    is used by the debug-transform command.
    """
    tab += 1

    if isinstance(msg, Model):
        msg = fromstring(msg.render())

    print('%s`- %s: %s %s' % (
        '  ' * tab,
        highlight(msg.tag, None, True),
        highlight(msg.text, 'red', False) if msg.text is not None else '',
        highlight(msg.attrib, 'green', True) if msg.attrib.keys() else ''
    ))
    for c in msg.getchildren():
        print('  %s`- %s: %s %s' % (
            '  ' * tab,
            highlight(c.tag, None, True),
            highlight(c.text, 'red', False) if c.text is not None else '',
            highlight(c.attrib, 'green', True) if c.attrib.keys() else ''
        ))
        for sc in c.getchildren():
            tab += 1
            console_writer(sc, tab)
            tab -= 1



from __future__ import print_function

import sys

from canari.commands.common import canari_main
from canari.maltego.utils import highlight
from canari.commands.framework import SubCommand

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.7'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help='Lists all the available canari commands.',
    description='Lists all the available canari commands.'
)
def list_commands(opts):
    cmds = canari_main.subparsers.choices
    for name, cmd in sorted(cmds.items()):
        print('%s - %s' % (highlight(name, 'green', True), cmd.description), file=sys.stderr)



from __future__ import print_function

import keyword
import os
import re
import sys

from mrbob.configurator import Configurator
from mrbob.rendering import jinja2_env

from canari.maltego.configuration import MaltegoEntity
from canari.maltego.message import Entity, StringEntityField
from canari.pkgutils.maltego import MtzDistribution
from canari.project import CanariProject
from canari.question import parse_bool
from canari.commands.common import canari_main
from canari.commands.framework import SubCommand, Argument

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = ['Nadeem Douba']

__license__ = 'GPLv3'
__version__ = '0.5'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

type_mapping = {
    'string': 'StringEntityField',
    'int': 'IntegerEntityField',
    'byte': 'IntegerEntityField',
    'char': 'IntegerEntityField',
    'boolean': 'BooleanEntityField',
    'float': 'FloatEntityField',
    'date': 'DateEntityField',
    'datetime': 'DateTimeEntityField',
    'timespan': 'TimeSpanEntityField',
    'color': 'ColorEntityField',
    'double': 'FloatEntityField'
}


def parse_args(args):
    project = CanariProject(args.out_path)

    if project.is_valid:
        args.out_path = project.common_dir
        args.out_file = project.entities_py
    else:
        args.out_path = project.root_dir
        args.out_file = os.path.join(args.out_path, 'entities.py')

    if os.path.exists(args.out_file) and not args.append and not \
            parse_bool('%r already exists. Are you sure you want to overwrite it?' % args.out_file, default=False):
        exit(-1)

    if not args.mtz_file:
        if not project.is_valid or not os.path.lexists(project.entities_mtz):
            print("Please specify a valid MTZ file.", file=sys.stderr)
            exit(-1)
        args.mtz_file = project.entities_mtz

    if args.maltego_entities:
        args.namespace.extend(args.exclude_namespace)
        args.exclude_namespace = []

    args.project = project

    return args


def normalize_fn(fn):
    # Get rid of starting underscores or numbers and bad chars for var names in python
    return re.sub(r'[^A-Za-z0-9]+', '_', re.sub(r'^[^A-Za-z]+', '', fn))



@SubCommand(
    canari_main,
    help='Converts Maltego entity definition files to Canari python classes. Excludes Maltego built-in entities by '
         'default.',
    description='Converts Maltego entity definition files to Canari python classes. Excludes Maltego built-in entities '
                'by default.'
)
@Argument(
    'out_path',
    metavar='[output path]',
    help='Which path to write the output to.',
    default=os.getcwd(),
    nargs='?'
)
@Argument(
    '--mtz-file',
    '-m',
    metavar='<mtzfile>',
    help='A *.mtz file containing an export of Maltego entities.',
    required=False
)
@Argument(
    '--exclude-namespace',
    '-e',
    metavar='<namespace>',
    help='Name of Maltego entity namespace to ignore. Can be defined multiple times.',
    required=False,
    action='append',
    default=['maltego', 'maltego.affiliation']
)
@Argument(
    '--namespace',
    '-n',
    metavar='<namespace>',
    help='Name of Maltego entity namespace to generate entity classes for. Can be defined multiple times.',
    required=False,
    action='append',
    default=[]
)
@Argument(
    '--maltego-entities',
    '-M',
    help="Generate entities belonging to the 'maltego' namespace.",
    default=False,
    action='store_true'
)
@Argument(
    '--append',
    '-a',
    help='Whether or not to append to the existing *.py file.',
    action='store_true',
    default=False
)
@Argument(
    '--entity',
    '-E',
    metavar='<entity>',
    help='Name of Maltego entity to generate Canari python class for.',
    required=False,
    action='append',
    default=[]
)
def generate_entities(args):
    opts = parse_args(args)

    mtz = MtzDistribution(opts.mtz_file)
    target = opts.out_path

    variables = opts.project.configuration['variables']

    entity_definitions = {}

    matcher = re.compile('(.+)\.([^.]+)$')

    for entity_file in mtz.entities:
        entity = MaltegoEntity.parse(mtz.read_file(entity_file))
        namespace, name = matcher.match(entity.id).groups()
        if namespace in opts.exclude_namespace:
            continue
        elif not opts.namespace or namespace in opts.namespace:
            entity_definitions[(namespace, name)] = entity

    entity_classes = []

    if opts.append:
        module = opts.project.entities_module
        for entity_class in dir(module):
            entity_class = getattr(module, entity_class)
            if isinstance(entity_class, type) and issubclass(entity_class, Entity) and entity_class is not Entity \
                    and (entity_class._namespace_, entity_class.__name__) not in entity_definitions:
                entity_classes.append(entity_class)

    def get_entity_field_class(v):
        if v == 'int':
            return 'IntegerEntityField'
        elif v == 'float':
            return 'FloatEntityField'
        elif v == 'boolean':
            return 'BooleanEntityField'
        elif v == 'timespan':
            return 'TimeSpanEntityField'
        elif v == 'datetime':
            return 'DateTimeEntityField'
        elif v == 'date':
            return 'DateEntityField'
        elif v == 'long':
            return 'LongEntityField'
        else:
            return 'StringEntityField'

    def get_property_name(v):
        v = v.replace('.', '_').replace('-', '_')
        return '%s_' % v if keyword.iskeyword(v) else v

    jinja2_env.filters['entity_properties'] = \
        lambda v: reversed([(p, getattr(v, p)) for p in dir(v) if isinstance(getattr(v, p), StringEntityField) and
                            not hasattr(Entity, p)])

    jinja2_env.filters['get_entity_field_class'] = get_entity_field_class

    jinja2_env.filters['get_property_name'] = get_property_name

    variables.update({
        'entity.definitions': entity_definitions,
        'entity.classes': entity_classes
    }, )

    configurator = Configurator(
            'canari.resources.templates:generate_entities',
            target,
            {'non_interactive': True},
            variables=variables
    )

    configurator.ask_questions()

    print('Generating entities for %r...' % variables['project.name'], file=sys.stderr)
    configurator.render()

    print('done!', file=sys.stderr)





from __future__ import print_function

import inspect
import signal
import sys
import os

from canari.maltego.entities import Unknown
from canari.maltego.message import MaltegoMessage, MaltegoTransformExceptionMessage, MaltegoException, Field

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'on_terminate',
    'message',
    'highlight',
    'croak',
    'guess_entity_type',
    'to_entity',
    'get_transform_version',
    'debug',
    'progress'
]


def on_terminate(func):
    """Register a signal handler to execute when Maltego forcibly terminates the transform."""
    def exit_function(*args):
        func()
        exit(0)
    signal.signal(signal.SIGTERM, exit_function)


def message(m, fd=sys.stdout):
    """Write a MaltegoMessage to stdout and exit successfully"""

    if sys.platform == 'win32':
        decoding = sys.stdout.encoding if sys.version_info[0] > 2 else 'cp1252'
        print(
            MaltegoMessage(message=m).render(fragment=True, encoding='utf-8').decode(decoding),
            file=fd
        )
    else:
        print(
            MaltegoMessage(message=m).render(fragment=True),
            file=fd
        )
    sys.exit(0)


def highlight(s, color, bold):
    """
    Internal API: Returns the colorized version of the text to be returned to a POSIX terminal. Not compatible with
    Windows (yet).
    """
    if os.name == 'posix':
        attr = []
        if color == 'green':
            # green
            attr.append('32')
        elif color == 'red':
            # red
            attr.append('31')
        else:
            attr.append('30')
        if bold:
            attr.append('1')
        s = '\x1b[%sm%s\x1b[0m' % (';'.join(attr), s)

    return s


def croak(error, message_writer=message):
    """Throw an exception in the Maltego GUI containing error_msg."""
    if isinstance(error, MaltegoException):
        message_writer(MaltegoTransformExceptionMessage(exceptions=[error]))
    else:
        message_writer(MaltegoTransformExceptionMessage(exceptions=[MaltegoException(error)]))


def guess_entity_type(transform_module, fields):
    """
    Internal API: Returns the entity type based on the following best match algorithm:

    1. If a transform does not specify the input entity types, the Unknown entity will be returned.
    2. If a transform only has one input entity type, then that entity type will be returned.
    3. If a transform has more than one input entity type, then the entity type that has the
       most number of matching entity fields in the entity's class definition will be returned.

    This is only used by the local transform runner to detect the input entity type since this information is excluded
    at run-time.
    """
    if not hasattr(transform_module.dotransform, 'inputs') or not transform_module.dotransform.inputs:
        return Unknown
    if len(transform_module.dotransform.inputs) == 1 or not fields:
        return transform_module.dotransform.inputs[0][1]
    num_matches = 0
    best_match = Unknown
    for category, entity_type in transform_module.dotransform.inputs:
        l = len(set(entity_type._fields_to_properties_.keys()).intersection(fields.keys()))
        if l > num_matches:
            num_matches = l
            best_match = entity_type
    return best_match


def to_entity(entity_type, value, fields):
    """
    Internal API: Returns an instance of an entity of type entity_type with the specified value and fields (stored in
    dict). This is only used by the local transform runner as a helper function.
    """
    e = entity_type(value)
    for k, v in fields.items():
        e.fields[k] = Field(k, v)
    return e


def get_transform_version(transform):
    """
    Internal API: Returns the version of the transform function based on the transform function's signature. Currently,
    only two versions are supported (2 and 3). This is what version 2 transform functions look like:

    def transform(request, response):
        ...

    Version 3 transforms have the additional config variable like so:

    def transform(request, response, config):
        ...

    Or can have a varargs parameter as a third argument:

    def transform(request, response, *args):
        ...

    In both cases, version 3 transforms will be passed a local copy of the canari configuration object as the third
    argument. However, in the latter example, the configuration object will be stored in a tuple (i.e. (config,)).
    """
    spec = inspect.getargspec(transform)
    if spec.varargs is not None:
        return 3
    n = len(spec.args)
    if 2 <= n <= 3:
        return n
    raise Exception('Could not determine transform version.')


def debug(*args):
    """Send debug messages to the Maltego console."""
    for i in args:
        sys.stderr.write('D:%s\n' % str(i))
        sys.stderr.flush()


def progress(i):
    """Send a progress report to the Maltego console."""
    sys.stderr.write('%%%d\n' % min(max(i, 0), 100))
    sys.stderr.flush()



from __future__ import print_function

import os
import sys

from canari.commands.common import canari_main
from canari.commands.framework import SubCommand, Argument
from canari.pkgutils.transform import TransformDistribution
from canari.utils.fs import PushDir

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.6'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


# Dictionary of detected transforms
transforms = {}


# Main
@SubCommand(
    canari_main,
    help="Loads a canari package into Plume.",
    description="Loads a canari package into Plume."
)
@Argument(
    'package',
    metavar='<package>',
    help='the name of the canari transforms package to load into Plume.'
)
@Argument(
    '-d',
    '--plume-dir',
    metavar='[www dir]',
    default=os.getcwd(),
    help='the path where Plume is installed.'
)
@Argument(
        '--accept-defaults',
        '-y',
        help='Load Plume package with all the defaults in non-interactive mode.',
        default=False,
        action='store_true'
)
def load_plume_package(opts):

    with PushDir(opts.plume_dir):
        if not os.path.exists('canari.conf'):
            print('Plume does not appear to be installed in %s.' % opts.plume_dir, file=sys.stderr)
            print("Please run 'canari install-plume' and try again.", file=sys.stderr)
            exit(-1)

        transform_package = None
        try:
            transform_package = TransformDistribution(opts.package)
        except ValueError as e:
            print('An error occurred', e, file=sys.stderr)
            exit(-1)

        if transform_package.has_remote_transforms:
            try:
                transform_package.configure(opts.plume_dir, remote=True, defaults=opts.accept_defaults)
            except ImportError as e:
                print('An error occurred while trying to import %r from %s: %s' % (
                    transform_package.name, opts.plume_dir, e
                ), file=sys.stderr)
                print('Please make sure that %r is importable from %s' % (transform_package.name, opts.plume_dir),
                      file=sys.stderr)
                exit(-1)
            print('Please restart plume for changes to take effect.', file=sys.stderr)
            exit(0)

    print('Error: no remote transforms found. '
          'Please make sure that at least one transform has remote=True set before retrying.', file=sys.stderr)
    exit(-1)



from future.standard_library import install_aliases

install_aliases()

import os
import sys
from io import FileIO

from tempfile import NamedTemporaryFile, gettempdir
from time import time

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = ['Andrew Udvare']

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'flock',
    'FileSemaphore',
    'FileMutex',
    'UniqueFile',
    'CookieFile',
    'PushDir'
]

if os.name == 'nt':
    import msvcrt
    from ctypes import *
    from ctypes.wintypes import BOOL, DWORD, HANDLE

    LOCK_SH = 0x0  # the default
    LOCK_NB = 0x1  # LOCKFILE_FAIL_IMMEDIATELY
    LOCK_EX = 0x2  # LOCKFILE_EXCLUSIVE_LOCK
    LOCK_UN = 0x4  # Unlock file. Not in NT API, just needs to be there.


    # --- the code is taken from pyserial project ---
    #
    # detect size of ULONG_PTR
    def is_64bit():
        return sizeof(c_ulong) != sizeof(c_void_p)


    if is_64bit():
        ULONG_PTR = c_int64
    else:
        ULONG_PTR = c_ulong
    PVOID = c_void_p


    # --- Union inside Structure by stackoverflow:3480240 ---
    class _OFFSET(Structure):
        _fields_ = [
            ('Offset', DWORD),
            ('OffsetHigh', DWORD)]


    class _OFFSET_UNION(Union):
        _anonymous_ = ['_offset']
        _fields_ = [
            ('_offset', _OFFSET),
            ('Pointer', PVOID)]


    class OVERLAPPED(Structure):
        _anonymous_ = ['_offset_union']
        _fields_ = [
            ('Internal', ULONG_PTR),
            ('InternalHigh', ULONG_PTR),
            ('_offset_union', _OFFSET_UNION),
            ('hEvent', HANDLE)]


    LPOVERLAPPED = POINTER(OVERLAPPED)

    # --- Define function prototypes for extra safety ---
    LockFileEx = windll.kernel32.LockFileEx
    LockFileEx.restype = BOOL
    LockFileEx.argtypes = [HANDLE, DWORD, DWORD, DWORD, DWORD, LPOVERLAPPED]
    UnlockFileEx = windll.kernel32.UnlockFileEx
    UnlockFileEx.restype = BOOL
    UnlockFileEx.argtypes = [HANDLE, DWORD, DWORD, DWORD, LPOVERLAPPED]


    def flock(file_, flags):
        hfile = msvcrt.get_osfhandle(file_.fileno())
        overlapped = OVERLAPPED()
        if flags & LOCK_UN and UnlockFileEx(hfile, 0, 0, 0xFFFF0000, byref(overlapped)):
            return
        elif (not flags or flags & (LOCK_EX | LOCK_NB | LOCK_SH)) and \
                LockFileEx(hfile, flags, 0, 0, 0xFFFF0000, byref(overlapped)):
            return
        raise IOError(GetLastError())

else:
    from fcntl import flock, LOCK_EX, LOCK_NB, LOCK_SH, LOCK_UN


class FileSemaphore(FileIO):

    def __init__(self, name, mode='rb', buffering=True):
        self.locked = False
        super(FileSemaphore, self).__init__(name, mode, buffering)

    def lockex(self, nb=False):
        flags = LOCK_EX
        if nb:
            flags |= LOCK_NB
        flock(self, flags)
        self.locked = True

    def locksh(self, nb=False):
        flags = LOCK_SH
        if nb:
            flags |= LOCK_NB
        flock(self, flags)
        self.locked = True

    def unlock(self, nb=False):
        flags = LOCK_UN
        if nb:
            flags |= LOCK_NB
        flock(self, flags)
        self.locked = False

    def __del__(self):
        if self.locked and not self.closed:
            self.unlock()

    def close(self):
        super(FileSemaphore, self).close()
        if self.locked and not self.closed:
            self.unlock()

    def __exit__(self, *args):
        super(FileSemaphore, self).__exit__(*args)
        if self.locked and not self.closed:
            self.unlock()


class FileMutex(FileSemaphore):

    def __init__(self, name):
        super(FileMutex, self).__init__(os.path.join(gettempdir(), name), 'w')
        self.lockex()


def UniqueFile(self, name, delete=False):
    n, e = os.path.splitext(name)
    f = NamedTemporaryFile(suffix=e, prefix='%s_' % n, delete=delete)
    if os.name == 'posix':
        os.chmod(f.name, 0o644)
    return f


class PushDir(object):
    """Ripped from here: https://gist.github.com/Tatsh/7131812"""

    def __init__(self, dir_name):
        self.cwd = os.path.realpath(dir_name)
        self.original_dir = None

    def __enter__(self):
        self.original_dir = os.getcwd()
        if os.path.lexists(self.cwd):
            os.chdir(self.cwd)
        else:
            sys.stderr.write('WARNING: %r directory does not exist. Falling back to %r.' %
                             (self.cwd, self.original_dir))
        return self

    def __exit__(self, type_, value, tb):
        os.chdir(self.original_dir)


class CookieFile(FileSemaphore):

    @property
    def age(self):
        return time() - os.stat(self.name).st_mtime

    @property
    def expired(self):
        return self.age > self.max_age

    def __init__(self, name, mode='rb', buffering=True, max_age=86400):
        name = os.path.join(gettempdir(), name)
        self.max_age = max_age
        super(CookieFile, self).__init__(name, mode, buffering=buffering)

        if 'w' in self.mode:
            self.lockex()
        elif 'r' in self.mode:
            self.locksh()
            if self.expired:
                self.close()
                raise IOError('Cookie file expired (age=%d > max_age=%d).' % (self.age, max_age))



import re

from canari.maltego.configuration import AuthenticationType
from canari.maltego.entities import Unknown

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'Transform'
]


def camel_to_title(s):
    return re.sub(
            '([A-Z]+)([A-Z][a-z])',
            r'\1 \2',
            re.sub(
                    '([a-z])([A-Z]+)',
                    r'\1 \2',
                    re.sub(
                        '([0-9]+)',
                        r' \1 ',
                        s
                    )
            )
    ).strip()


class Transform(object):

    # Specifies the author of the transform. If not specified __author__ will be used for the create-profile command.
    author = ''

    # A detailed description of the transform. The description can be read from the Maltego Transforms dialog box. If
    # this is not set, the transform class' doc comments will be used.
    description = ''

    # The label found in the transform context menu when right clicking on an entity.
    display_name = ''

    # Specifies the help URL for the specified transform.
    help_url = ''

    # The unique identifier of the transform. Should be in reverse dotted format similar to Java. For example,
    # sploitego.FastNmap. If none is specified, the name will be set to the module dot class name.
    name = ''

    # The Maltego input entity type.
    input_type = Unknown

    # The Maltego transform set name.
    transform_set = ''

    # Specifies whether or not the transform is deprecated.
    deprecated = False

    # Specifies whether or not the transform can be run on an application server (remotely).
    remote = False

    # Specifies whether or not the transform should open a debugging window in Maltego when executed.
    debug = False

    # Specifies whether or not the transform requires privileged user access.
    superuser = False

    # Specifies the type of authentication used by this transform. Can be either 'none', 'mac', or 'license'.
    authentication = AuthenticationType.Anonymous

    # Specifies a disclaimer for the transform that appears prior to the first execution of a transform
    disclaimer = ''

    # Reserved for future external Maltego transforms support
    command = None

    def __init__(self):
        if not self.name:
            self.name = '.'.join([self.__module__.split('.', 1)[0], self.__class__.__name__])
        if not self.display_name:
            self.display_name = camel_to_title(self.__class__.__name__)
        if not self.description and self.__doc__:
            self.description = self.__doc__
        if not self.transform_set:
            self.transform_set = self.__module__.split('.', 1)[0].title()

    def do_transform(self, request, response, config):
        """
        This is the main entry point for the transform. Parameters and transform fields are passed via the config
        variable. The input entity is passed via the request object. Once a transform has executed its data mining
        operations, values should be returned via the response object. Additional methods can be defined as long as
        they don't overwrite the built-in property names: author, description, display_name, help_url, name, deprecated,
        remote, debug, and superuser.

        :param request: a MaltegoTransformRequest object.
        :param response: a MaltegoTransformResponse object.
        :param config: a CanariConfig object.
        :return: The response object should be returned.

        :Example:

        class GoogleTransform(MaltegoTransform):
            # ...
            def do_transform(self, request, response, config):
                term = request.value
                results = google(term)
                for result in results:
                    response += WebSite(result)
                return response
        """
        raise NotImplementedError("The 'do_transform' method needs to be implemented!")

    def on_terminate(self):
        """
        This method gets called when the transform's execution is prematurely terminated. It is only applicable for
        local transforms.
        """
        pass


#!/usr/bin/env python
from __future__ import print_function
from past.builtins import basestring
from six import b

import sys

if sys.version_info[0] > 2:
    from configparser import NoSectionError
else:
    # noinspection PyUnresolvedReferences
    from ConfigParser import NoSectionError

# Builtin imports
import logging
import os
import tempfile
import traceback
from hashlib import md5
from logging.handlers import RotatingFileHandler

from flask import Flask, Response, request, safe_join

import canari.resource
from canari.commands.common import fix_binpath, fix_pypath
from canari.config import load_config, OPTION_REMOTE_PATH
from canari.maltego.entities import Phrase, Unknown
from canari.maltego.message import (MaltegoMessage, MaltegoTransformResponseMessage, MaltegoTransformExceptionMessage,
                                    MaltegoException)
from canari.maltego.transform import Transform
from canari.mode import set_canari_mode, CanariMode
from canari.pkgutils.transform import TransformDistribution

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.5'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'application'
]

# Initialize Canari modes and bin path
set_canari_mode(CanariMode.RemotePlumeDispatch)
fix_binpath(load_config()[OPTION_REMOTE_PATH])
fix_pypath()


def get_image_path(i):
    return os.path.join('static', md5(b(i)).hexdigest())


def get_image_url(i):
    return '%s/static/%s' % (request.host_url, md5(b(i)).hexdigest())


# Monkey patch our resource lib to automatically rewrite icon urls
_icon_resource = canari.resource.icon_resource
canari.resource.icon_resource = lambda name, pkg=None: get_image_url(_icon_resource(name, pkg))

_calling_package = canari.resource.calling_package
canari.resource.calling_package = lambda frame=4: _calling_package(frame)

# Use temporary directory as Python Egg Cache
os.environ['PYTHON_EGG_CACHE'] = tempfile.gettempdir()


class Version(Transform):
    input_type = Phrase

    def do_transform(self, r, res, config):
        if r.entity.value == 'version':
            res += Phrase('Canari v%s' % __version__)
        return res


class Plume(Flask):
    four_o_four = 'Whaaaaat?'

    def __init__(self, import_name, *args, **kwargs):
        super(Plume, self).__init__(import_name, *args, **kwargs)
        self.transforms = {}
        self.resources = []
        self._initialize()

    def _copy_images(self, pkg):
        if pkg.endswith('.transforms'):
            pkg = pkg.replace('.transforms', '')
        for i in canari.resource.image_resources(pkg):
            img_name = get_image_path(i)
            self.resources.append(img_name)
            if not os.path.exists(img_name):
                print('Copying %s to %s...' % (i, img_name), file=sys.stderr)
                with open(i, mode='rb') as src:
                    with open(img_name, mode='wb') as dst:
                        dst.write(src.read())

    def _initialize(self):

        packages = None

        # Read packages that are to be loaded at runtime
        try:
            config = load_config()
            packages = config['canari.remote.packages']
        except NoSectionError:
            print('Exiting... You did not specify a [canari.remote] section and a "packages" '
                  'option in your canari.conf file!', file=sys.stderr)
            exit(-1)

        # Is packages not blank
        if not packages:
            print(
                'Exiting... You did not specify any transform packages to load in your canari.conf file!',
                file=sys.stderr)
            exit(-1)
        elif isinstance(packages, basestring):
            packages = [packages]

        # Create the static directory for static file loading
        if not os.path.exists('static'):
            os.mkdir('static', 0o755)

        # Iterate through the list of packages to load
        for p in packages:
            # Copy all the image resource files in case they are used as entity icons

            distribution = TransformDistribution(p)

            print('Loading transform package %s' % repr(p), file=sys.stderr)

            for transform in distribution.remote_transforms:
                transform_name = transform().name
                print('Loading transform %s at /%s...' % (repr(transform_name), transform_name), file=sys.stderr)
                if os.name == 'posix' and transform.superuser and os.geteuid() and __name__.startswith('_mod_wsgi_'):
                    print('WARNING: mod_wsgi does not allow applications to run with root privileges. '
                          'Transform %s ignored...' % repr(transform_name), file=sys.stderr)
                    continue

                self.transforms[transform_name] = transform

            self._copy_images(p)

        if not self.transforms:
            print('Exiting... Your transform packages have no remote transforms.', file=sys.stderr)
            exit(-1)

        self.transforms['canari.Version'] = Version


# Create our Flask app.
application = Plume(__name__)


def croak(cause):
    """Throw an exception in the Maltego GUI containing cause.

    :param cause: a string containing the issue description.
    """
    return MaltegoMessage(
        message=MaltegoTransformExceptionMessage(
            exceptions=[
                MaltegoException(cause)
            ]
        )
    ).render()


def message(msg):
    """Write a MaltegoMessage to stdout and exit successfully"""
    v = MaltegoMessage(message=msg).render()
    return Response(v, status=200, mimetype='text/xml')


def do_transform(transform):
    try:
        # Let's get an XML object tree
        req = MaltegoMessage.parse(request.data).message

        # If our transform define an input entity type then we should check
        # whether the request contains the right type
        if transform.input_type and transform.input_type is not Unknown and \
                not isinstance(req.entity, transform.input_type):
            return Response(application.four_o_four, status=404)

        # Execute it!
        msg = transform().do_transform(
            req,
            MaltegoTransformResponseMessage(),
            load_config()
        )

        # Let's serialize the return response and clean up whatever mess was left behind
        if isinstance(msg, MaltegoTransformResponseMessage):
            return message(msg)
        else:
            raise MaltegoException(str(msg))

    # Unless we croaked somewhere, then we need to fix things up here...
    except MaltegoException as me:
        return croak(str(me))
    except Exception:
        if application.debug:
            return croak(traceback.format_exc())
        else:
            return croak('Transform execution failed.')


# This is where the TDS will ask: "Are you a transform?" and we say "200 - Yes I am!" or "404 - PFO"
@application.route('/<transform_name>', methods=['GET'])
def transform_checker(transform_name):
    if transform_name in application.transforms:
        return Response('Yes?', status=200)
    return Response(application.four_o_four, status=404)


@application.route('/static/<resource_name>', methods=['GET'])
def static_fetcher(resource_name):
    resource_name = safe_join('static', resource_name)
    if resource_name in application.resources:
        return Response(open(resource_name, mode='rb').read(), status=200, mimetype='application/octet-stream')
    return Response(application.four_o_four, status=404)


# This is where we process a transform request.
@application.route('/<transform_name>', methods=['POST'])
def transform_runner(transform_name):
    if transform_name not in application.transforms:
        return Response(application.four_o_four, status=404)
    return do_transform(application.transforms[transform_name])


# To run Flask standalone just type `python -m canari.tas.plume`
if __name__ == '__main__':
    handler = RotatingFileHandler('plume.log', maxBytes=10000, backupCount=1)
    handler.setLevel(logging.INFO)
    application.logger.addHandler(handler)
    application.run(debug=True)



from safedexml import Model, fields

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'MaltegoElement',
    'fields'
]


class MaltegoElement(Model):
    """MaltegoElement is the base element for all XML elements defined in the Maltego DTD."""

    class meta:
        order_sensitive = False

    def __iadd__(self, other):
        return self

    def __add__(self, other):
        return self.__iadd__(other)


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '3.3.0'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'maltego',
    'resources',
    'utils',
    'config',
    'easygui',
    'framework',
    'mode',
    'resource',
    'tas',
    'entrypoints'
]




from __future__ import print_function

import os
import sys

from mrbob.configurator import Configurator
from mrbob.parsing import parse_config

from canari.project import CanariProject
from canari.commands.common import canari_main
from canari.commands.framework import SubCommand, Argument

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.4'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def parse_args(args):
    if args.transform in ['common', 'common.py']:
        print("Error: 'common' is a reserved module. Please name your transform something else.", file=sys.stderr)
        exit(-1)
    return args


@SubCommand(
    canari_main,
    help='Creates a new transform in the specified directory and auto-updates __init__.py.',
    description='Creates a new transform in the specified directory and auto-updates __init__.py.'
)
@Argument(
    'transform',
    metavar='<transform name>',
    help='The name of the transform you wish to create.'
)
def create_transform(args):

    opts = parse_args(args)
    project = CanariProject()

    transform_module = (opts.transform if not opts.transform.endswith('.py') else opts.transform[:-3])
    transform_name = ''.join([i[0].upper()+i[1:] for i in transform_module.split('_')])
    transform_module = transform_module.lower()

    if '.' in transform_module:
        print("Transform name (%r) cannot have a dot ('.')." % transform_name, file=sys.stderr)
        exit(-1)
    elif not transform_module:
        print("You must specify a valid transform name.", file=sys.stderr)
        exit(-1)

    target = project.root_dir
    transform_directory = project.transforms_dir

    if os.path.exists(os.path.join(transform_directory, '%s.py' % transform_module)):
        print('Transform %r already exists... quitting' % transform_module, file=sys.stderr)
        exit(-1)

    variables = parse_config(os.path.join(target, '.mrbob.ini'))['variables']

    variables.update({'transform.module': transform_module, 'transform.name': transform_name})

    configurator = Configurator(
        'canari.resources.templates:create_transform',
        target,
        {'non_interactive': True},
        variables=variables
    )

    configurator.ask_questions()

    print('Creating transform %r...' % transform_module, file=sys.stderr)
    configurator.render()

    print('done!', file=sys.stderr)



from __future__ import print_function

import importlib
import os
import string
import sys
from importlib import import_module
from pkgutil import iter_modules

from mrbob.configurator import Configurator
from past.builtins import basestring
from pkg_resources import resource_listdir, resource_filename

from canari.config import CanariConfigParser, OPTION_LOCAL_CONFIGS, SECTION_LOCAL, OPTION_REMOTE_PACKAGES, \
    SECTION_REMOTE, OPTION_REMOTE_CONFIGS
from canari.maltego.message import EntityTypeFactory
from canari.maltego.transform import Transform
from canari.pkgutils.maltego import MaltegoDistribution, MtzDistribution
from canari.question import parse_bool
from canari.utils.fs import PushDir

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = ['Nadeem Douba']

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'TransformDistribution'
]

INCOMPATIBLE = """
!!!!!!!!!!!!!!!!!!!!!!!!!!! ERROR: NOT SUPPORTED !!!!!!!!!!!!!!!!!!!!!!!!!!!

 Starting from Maltego Radium (v3.4.0) the 'canari install-package' command
 is no longer supported. Please use the 'canari create-profile' command,
 instead. This will create an importable config file (*.mtz) which can be
 imported using the 'Import Configuration' option in Maltego. This option
 can be found by clicking on the <Maltego icon> in the top left corner of
 your Maltego window then scrolling to 'Import' then 'Import Configuration'.

 NOTE: This command will automatically install and configure the
 'canari.conf' file for you in the default location for your OS.

 EXAMPLE:

 shell> canari create-profile sploitego
 ...
 shell> ls
 sploitego.mtz <--- Import this file

!!!!!!!!!!!!!!!!!!!!!!!!!!! ERROR: NOT SUPPORTED !!!!!!!!!!!!!!!!!!!!!!!!!!!
"""


class TransformDistribution(object):
    def __init__(self, package_name):
        self._package_name = package_name.replace('.transforms', '') \
            if package_name.endswith('.transforms') else package_name

        self._package = import_module(self.name)

        print('Looking for transforms in %s...' % self.name, file=sys.stderr)
        try:
            self.import_package(self._package)
        except ImportError as e:
            raise ImportError("Does not appear to be a valid canari package. "
                              "Couldn't import the '%s.transforms' package in '%s'. Error message: %s" %
                              (self.name, self.name, e))

        self._transforms = list(self.find_all_subclasses(Transform))
        self._remote_transforms = [t for t in self._transforms if t.remote]
        self._config = '%s.conf' % self.name
        self._resources = '%s.resources' % self.name
        self._package_path = os.path.abspath(self._package.__path__[0]
                                             if isinstance(self._package.__path__, list)
                                             else self._package.__path__._path[0])
        self._default_prefix = os.path.join(os.path.expanduser('~'), '.canari') if self.is_site_package else os.getcwd()
        self._entities = list({v for v in EntityTypeFactory.registry.values()
                               if v.__module__.startswith(self._package_name)})
        self._author = getattr(self._package, '__author__', '')
        self._email = getattr(self._package, '__email__', '')
        try:
            self._machines = [
                m for m in resource_listdir(self.get_resource_module('maltego'), '') if m.endswith('.machine')]
        except ImportError:
            self._machines = []
        if self._package_name != 'canari' and not self.has_transforms:
            raise ValueError('Error: no transforms found...')
        print('Package loaded.', file=sys.stderr)

    @property
    def author(self):
        return self._author

    @property
    def author_email(self):
        return self._email

    @property
    def entities(self):
        return self._entities

    @property
    def name(self):
        return self._package_name

    @property
    def package_path(self):
        return self._package_path

    @property
    def transforms(self):
        return self._transforms

    @property
    def remote_transforms(self):
        return self._remote_transforms

    @property
    def has_remote_transforms(self):
        return bool(self.remote_transforms)

    @property
    def has_transforms(self):
        return bool(self.transforms)

    @property
    def config_file(self):
        return self._config

    @property
    def resources(self):
        return self._resources

    @property
    def is_site_package(self):
        for p in sys.path:
            if p and self.package_path.startswith(p):
                return True
        return False

    @property
    def machines(self):
        return self._machines

    @property
    def entities_file(self):
        return resource_filename(self.get_resource_module('maltego'), 'entities.mtz')

    @property
    def profile_file(self):
        return resource_filename(self.get_resource_module('maltego'), 'profile.mtz')

    @property
    def default_prefix(self):
        return self._default_prefix

    def get_resource_module(self, name):
        return '%s.%s' % (self._resources, name)

    @staticmethod
    def _write_config(src, dst, defaults=False, **kwargs):
        if not TransformDistribution._check_file_exists(dst, defaults):
            return

        print('Writing %s to %s' % (src, dst), file=sys.stderr)
        with open(dst, mode='w') as w:
            if kwargs.pop('is_template', None):
                w.write(
                        string.Template(
                                open(src).read()
                        ).substitute(**kwargs)
                )
            else:
                w.write(
                        open(src).read()
                )

    @classmethod
    def _is_package_loaded(cls, pkg):
        return pkg in sys.modules

    @classmethod
    def import_package(cls, pkg):
        prefix = pkg.__name__

        if prefix == 'canari' and not cls._is_package_loaded('canari.maltego.entities'):
            pass
        else:
            for _, module_name, is_pkg in iter_modules(pkg.__path__):
                module_name = '.'.join([prefix, module_name])
                if not cls._is_package_loaded(module_name):
                    pkg = importlib.import_module(module_name)
                else:
                    pkg = sys.modules[module_name]
                if is_pkg:
                    TransformDistribution.import_package(pkg)

    def find_all_subclasses(self, cls):
        for subclass in cls.__subclasses__():
            if subclass.__module__.startswith(self._package_name) and subclass.input_type:
                yield subclass
            for sub_subclass in self.find_all_subclasses(subclass):
                yield sub_subclass

    def _update_config(self, canari_config, load=True, remote=False, **kwargs):

        with PushDir(os.path.dirname(canari_config)):

            config = CanariConfigParser()
            config.read(canari_config)

            configs_option = OPTION_REMOTE_CONFIGS if remote else OPTION_LOCAL_CONFIGS
            config_section = SECTION_REMOTE if remote else SECTION_LOCAL

            if configs_option not in config:
                if config_section not in config:
                    config.add_section(config_section)
                config[configs_option] = ''

            configs = config.get_as_list(configs_option)

            if load:
                for (k, v) in kwargs.get('additional_options', {}).items():
                    config['.'.join([config_section, k])] = v

                if self.config_file not in configs:
                    print('Updating %s...' % canari_config, file=sys.stderr)
                    configs.append(self.config_file)
                    config[configs_option] = configs

                if remote:
                    packages = config.get_as_list(OPTION_REMOTE_PACKAGES)
                    if self.name not in packages:
                        packages.append(self.name)
                        config[OPTION_REMOTE_PACKAGES] = packages
            else:
                if self.config_file in configs:
                    print('Updating %s...' % canari_config, file=sys.stderr)
                    configs.remove(self.config_file)
                    config[configs_option] = configs

                if remote:
                    packages = config.get_as_list(OPTION_REMOTE_PACKAGES)
                    if self.name in packages:
                        packages.remove(self.name)
                        config[OPTION_REMOTE_PACKAGES] = packages

            config.write(open(canari_config, mode='w'))

    @staticmethod
    def _check_file_exists(dst, defaults=False):
        return not os.path.lexists(dst) or \
               (not defaults and parse_bool('%s already exists. Would you like to overwrite it?' % dst, default=False))

    def configure(self, install_prefix, load=True, remote=False, defaults=False, **kwargs):
        dst_canari_conf = os.path.join(install_prefix, 'canari.conf')
        if load and self._check_file_exists(dst_canari_conf, defaults):
            print('Writing fresh copy of canari.conf to %r...' % dst_canari_conf, file=sys.stderr)
            variables = {
                'canari.command': ' '.join(sys.argv),
                'profile.config': self.config_file if self.name != 'canari' else '',
                'profile.path': '${PATH},/usr/local/bin,/opt/local/bin' if os.name == 'posix' else ''
            }

            configurator = Configurator('canari.resources.templates:create_profile',
                                        install_prefix,
                                        {'non_interactive': True, 'remember_answers': True},
                                        variables=variables)
            configurator.render()

        if self._package_name != 'canari':
            dst_package_conf = os.path.join(install_prefix, self.config_file)
            if load:
                print('Copying %r to %r...' % (self.config_file, dst_package_conf))
                package_config = resource_filename(self.get_resource_module('etc'), self.config_file)
                self._write_config(package_config, dst_package_conf, defaults)
            self._update_config(dst_canari_conf, load, remote, **kwargs)

    def _init_install_prefix(self, install_prefix):
        if not install_prefix:
            install_prefix = self.default_prefix
        if not os.path.lexists(install_prefix):
            os.makedirs(install_prefix)
        return install_prefix

    def install(self, install_prefix, distribution, configure=True, is_remote=False):
        if isinstance(distribution, basestring) or not distribution:
            distribution = MaltegoDistribution(distribution)
        if not isinstance(distribution, MtzDistribution):
            if distribution.version >= '3.4.0':
                raise ValueError(INCOMPATIBLE)
            print('Installing transform package %s...' % self.name, file=sys.stderr)

        install_prefix = self._init_install_prefix(install_prefix)

        self._install_transforms(install_prefix, distribution)
        self._install_entities(distribution)
        self._install_machines(distribution)

        if configure:
            self.configure(install_prefix, remote=is_remote)

    def _install_transforms(self, prefix, distribution):
        for transform in self.transforms:
            distribution.add_transform(prefix, 'Local', transform, server='Local')

    def _install_entities(self, distribution):
        try:
            src = self.entities_file
            if not os.path.lexists(src):
                return
            mtz = MtzDistribution(src)

            for entity_file in mtz.entity_files:
                distribution.add_entity(mtz.read_file(entity_file))

            for icon_category in mtz.icon_categories:
                icon_category_dir = distribution.get_icon_category_dir(icon_category)
                for icon in mtz.get_icons_by_category(icon_category):
                    data = mtz.read_file(icon)
                    p = distribution.path_join(icon_category_dir, os.path.basename(icon))
                    print('Installing custom icon %s to %s...' % (icon, p), file=sys.stderr)
                    distribution.write_file(p, data)

        except ImportError:
            pass

    def _install_machines(self, distribution):
        try:
            package = self.get_resource_module('maltego')
            for machine in self.machines:
                src = resource_filename(package, machine)
                distribution.add_machine(machine, open(src).read())
        except ImportError:
            pass

    def uninstall(self, install_prefix, maltego_prefix=None):
        distribution = MaltegoDistribution(maltego_prefix)
        if distribution.version >= '3.4.0':
            raise ValueError("""
!!!!!!!!!!!!!!!!!!!!!!!!!!! ERROR: NOT SUPPORTED !!!!!!!!!!!!!!!!!!!!!!!!!!!

 Starting from Maltego Radium (v3.4.0) the 'canari uninstall-package'
 command is no longer supported. Please use the Maltego interface to
 uninstall packages.

!!!!!!!!!!!!!!!!!!!!!!!!!!! ERROR: NOT SUPPORTED !!!!!!!!!!!!!!!!!!!!!!!!!!!
            """)

        install_prefix = self._init_install_prefix(install_prefix)

        self._uninstall_transforms(distribution)
        self._uninstall_machines(distribution)
        self.configure(install_prefix, load=False)

    def _uninstall_machines(self, distribution):
        try:
            for machine in self.machines:
                distribution.remove_machine(machine)
        except ImportError:
            pass

    def _uninstall_transforms(self, distribution):
        for transform in self.transforms:
            distribution.remove_transform('Local', transform, server='Local')

    def create_profile(self, install_prefix, current_dir, configure=True):
        mtz = os.path.join(current_dir, '%s.mtz' % self.name)
        print('Creating profile %s...' % mtz, file=sys.stderr)
        mtz = MtzDistribution(mtz, 'w')
        self.install(install_prefix, mtz, configure)
        mtz.close()

        print("""
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% SUCCESS! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 Successfully created %s. You may now import this file into
 Maltego.

 INSTRUCTIONS:
 -------------
 1. Open Maltego.
 2. Click on the home button (Maltego icon, top-left corner).
 3. Click on 'Import'.
 4. Click on 'Import Configuration'.
 5. Follow prompts.
 6. Enjoy!

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% SUCCESS! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            """ % mtz.filename)



from future.standard_library import install_aliases
from past.builtins import basestring

install_aliases()

import sys
import os
import re
import string

if sys.version_info[0] > 2:
    from configparser import ConfigParser, NoOptionError, NoSectionError, BasicInterpolation
else:
    # noinspection PyUnresolvedReferences
    from ConfigParser import SafeConfigParser, NoOptionError, NoSectionError

from urllib.parse import urlparse

from canari.mode import is_local_exec_mode, is_remote_exec_mode
from canari.utils.fs import PushDir
from canari.resource import global_config
from canari.utils.wordlist import wordlist

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'CanariConfigParser',
    'load_config',
    'NoOptionError',
    'NoSectionError',
    'SECTION_LOCAL',
    'SECTION_REMOTE',
    'OPTION_LOCAL_CONFIGS',
    'OPTION_LOCAL_PATH',
    'OPTION_REMOTE_CONFIGS',
    'OPTION_REMOTE_PACKAGES',
    'OPTION_REMOTE_PATH'
]

SECTION_LOCAL = 'canari.local'

SECTION_REMOTE = 'canari.remote'

OPTION_LOCAL_CONFIGS = 'canari.local.configs'
OPTION_REMOTE_CONFIGS = 'canari.remote.configs'

OPTION_LOCAL_PATH = 'canari.local.path'
OPTION_REMOTE_PATH = 'canari.remote.path'

OPTION_REMOTE_PACKAGES = 'canari.remote.packages'


def _interpolate_environment_variables(value):
    if isinstance(value, str):
        value = string.Template(value).safe_substitute(os.environ)
    return value


if sys.version_info[0] > 2:
    class CustomInterpolation(BasicInterpolation):

        def before_get(self, parser, section, option, value, defaults):
            return super().before_get(parser, section, option, _interpolate_environment_variables(value), defaults)


    class CanariConfigParser(ConfigParser):

        def __init__(self, *args, **kwargs):
            super(CanariConfigParser, self).__init__(*args, interpolation=CustomInterpolation(), **kwargs)

        def __iadd__(self, other):
            self.add_section(other)
            return self

        def __isub__(self, other):
            self.remove_section(other)
            return self

        def _interpolate(self, section, option, value, d):
            return SafeConfigParser._interpolate(self, section, option, _interpolate_environment_variables(value), d)

        def _parse_value(self, value):
            if value.startswith('object://') and is_local_exec_mode():
                r = urlparse(value)
                try:
                    v = r.path.lstrip('/')
                    m = __import__(r.netloc, globals(), locals(), [v])
                    value = m.__dict__[v]
                except ImportError:
                    pass
            elif re.match(r'^\d+$', value):
                value = int(value)
            elif re.match(r'^\d+\.\d+$', value):
                value = float(value)
            elif re.search(r'\s*(?<=[^\\]),+\s*', value):
                l = re.split(r'\s*(?<=[^\\]),+\s*', value)
                value = []
                for v in l:
                    value.append(self._parse_value(v))
            else:
                value = value.replace(r'\,', ',')
            return value

        def _render_value(self, value):
            if isinstance(value, str):
                if value.strip().startswith('object://'):
                    raise ValueError('Cannot set object option.')
                value = value.replace(',', '\\,')
            elif isinstance(value, list):
                value = ','.join([self._render_value(v) for v in value])
            elif callable(value):
                raise ValueError('Cannot set object option.')
            else:
                value = str(value)
            return value

        def _get_option_value(self, key, raise_on_empty_option=False):
            if not key and raise_on_empty_option:
                raise KeyError("Invalid key %r, must be in '<section>.<option>'" % key)
            if self.has_section(key):
                return key, ''
            return key.rsplit('.', 1) if '.' in key else (key, '')

        def __getitem__(self, key):
            section, option = self._get_option_value(key, True)
            value = self.get(section, option)
            value = self._parse_value(value)
            if option == 'wordlist':
                value = wordlist(value)
            return value

        def __setitem__(self, key, value):
            section, option = self._get_option_value(key, True)
            if not self.has_section(section):
                self.add_section(section)
            self.set(section, option, self._render_value(value))

        def __delitem__(self, key):
            section, option = self._get_option_value(key)
            if option:
                self.remove_option(section, option)
            else:
                self.remove_section(section)

        def __contains__(self, key):
            section, option = self._get_option_value(key)
            if option:
                return self.has_option(section, option)
            return self.has_section(section)

        def get_as_list(self, key):
            opt = self.__getitem__(key)
            if isinstance(opt, basestring):
                opt = [opt] if opt else []
            return opt

        def update(self, other, **kwargs):
            if not isinstance(other, CanariConfigParser):
                raise ValueError('Expected a CanariConfigParser, got %r instead' % type(other).__name__)
            self._sections.update(other._sections, **kwargs)


else:
    class CanariConfigParser(SafeConfigParser):

        def __iadd__(self, other):
            self.add_section(other)
            return self

        def __isub__(self, other):
            self.remove_section(other)
            return self

        def _interpolate(self, section, option, value, d):
            return SafeConfigParser._interpolate(self, section, option, _interpolate_environment_variables(value), d)

        def _parse_value(self, value):
            if value.startswith('object://') and is_local_exec_mode():
                r = urlparse(value)
                try:
                    v = r.path.lstrip('/')
                    m = __import__(r.netloc, globals(), locals(), [v])
                    value = m.__dict__[v]
                except ImportError:
                    pass
            elif re.match(r'^\d+$', value):
                value = int(value)
            elif re.match(r'^\d+\.\d+$', value):
                value = float(value)
            elif re.search(r'\s*(?<=[^\\]),+\s*', value):
                l = re.split(r'\s*(?<=[^\\]),+\s*', value)
                value = []
                for v in l:
                    value.append(self._parse_value(v))
            else:
                value = value.replace(r'\,', ',')
            return value

        def _render_value(self, value):
            if isinstance(value, str):
                if value.strip().startswith('object://'):
                    raise ValueError('Cannot set object option.')
                value = value.replace(',', '\\,')
            elif isinstance(value, list):
                value = ','.join([self._render_value(v) for v in value])
            elif callable(value):
                raise ValueError('Cannot set object option.')
            else:
                value = str(value)
            return value

        def _get_option_value(self, key, raise_on_empty_option=False):
            if not key and raise_on_empty_option:
                raise KeyError("Invalid key %r, must be in '<section>.<option>'" % key)
            if self.has_section(key):
                return key, ''
            return key.rsplit('.', 1) if '.' in key else (key, '')

        def __getitem__(self, key):
            section, option = self._get_option_value(key, True)
            value = self.get(section, option)
            value = self._parse_value(value)
            if option == 'wordlist':
                value = wordlist(value)
            return value

        def __setitem__(self, key, value):
            section, option = self._get_option_value(key, True)
            if not self.has_section(section):
                self.add_section(section)
            self.set(section, option, self._render_value(value))

        def __delitem__(self, key):
            section, option = self._get_option_value(key)
            if option:
                self.remove_option(section, option)
            else:
                self.remove_section(section)

        def __contains__(self, key):
            section, option = self._get_option_value(key)
            if option:
                return self.has_option(section, option)
            return self.has_section(section)

        def get_as_list(self, key):
            opt = self.__getitem__(key)
            if isinstance(opt, basestring):
                opt = [opt] if opt else []
            return opt

        def update(self, other):
            if not isinstance(other, CanariConfigParser):
                raise ValueError('Expected a CanariConfigParser, got %r instead' % type(other).__name__)
            self._sections.update(other._sections)


def load_config(config_file=None, recursive_load=True):
    if not config_file:
        config_file = os.path.join(os.getcwd(), 'canari.conf')
        if not os.path.lexists(config_file):
            config_file = os.path.join(os.path.expanduser('~'), '.canari', 'canari.conf')

    with PushDir(os.path.dirname(config_file)):
        config_parser = CanariConfigParser()
        config_parser.read([global_config, config_file])
        if recursive_load:
            if is_remote_exec_mode() and OPTION_REMOTE_CONFIGS in config_parser:
                config_parser.read(config_parser[OPTION_REMOTE_CONFIGS])
            elif OPTION_LOCAL_CONFIGS in config_parser:
                config_parser.read(config_parser[OPTION_LOCAL_CONFIGS])

        return config_parser



import os
import sys


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = ['Nadeem Douba']

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'find_pysudo',
    'find_dispatcher',
    'find_canari'
]


def find_script(name):
    ext = ''
    if sys.platform == 'win32':
        ext = '.bat'
    return os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), name + ext)


def find_dispatcher():
    return find_script('dispatcher')


def find_canari():
    return find_script('canari')


def find_pysudo():
    return find_script('pysudo')




from __future__ import print_function

import os
import sys

from canari.pkgutils.transform import TransformDistribution
from canari.utils.fs import PushDir

from canari.commands.common import canari_main
from canari.commands.framework import SubCommand, Argument


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.6'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help="Unloads a canari package from Plume.",
    description="Unloads a canari package from Plume."
)
@Argument(
    'package',
    metavar='<package>',
    help='the name of the canari transforms package to unload from Plume.'
)
@Argument(
    '-d',
    '--plume-dir',
    metavar='[www dir]',
    default=os.getcwd(),
    help='the path where Plume is installed.'
)
def unload_plume_package(opts):
    with PushDir(opts.plume_dir):
        if not os.path.exists('canari.conf'):
            print('Plume does not appear to be installed in %s.' % opts.plume_dir, file=sys.stderr)
            print("Please run 'canari install-plume' and try again.", file=sys.stderr)
            exit(-1)
        try:
            TransformDistribution(opts.package).configure(opts.plume_dir, load=False, remote=True)
        except ImportError:
            pass

    exit(0)


from __future__ import print_function

import os
import subprocess

import re
import sys

from canari import question, __version__ as version
from mrbob.configurator import Configurator

from canari.project import CanariProject
from canari.utils.fs import PushDir
from canari.commands.common import canari_main
from canari.commands.framework import SubCommand, Argument
from distutils.spawn import find_executable

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2016, Canari Project'
__credits__ = ['Nadeem Douba']

__license__ = 'GPLv3'
__version__ = '0.2'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def run_command(args, **kwargs):
    print('Running command: %s' % ' '.join(args), file=sys.stderr)
    return subprocess.Popen(args, **kwargs)


def get_output(process):
    return process.communicate()[0].decode('utf-8')


@SubCommand(
    canari_main,
    help='Creates a Docker build file pre-configured with Plume.',
    description='Creates a Docker build file that can be used to run remote transforms using Plume.'
)
@Argument(
    '-H',
    '--host',
    metavar='[host]',
    action='append',
    default=[],
    help='Docker daemon socket(s) to connect to'
)
@Argument(
    '-O',
    '--os',
    metavar='[container OS]',
    choices=['alpine', 'ubuntu', 'kalilinux'],
    default='alpine'
)
def dockerize_package(args):
    if sys.version_info[0] > 2:
        args.os += '-py3'

    project = CanariProject()

    print('Dockerizing %s transform package...' % project.name, file=sys.stderr)

    configurator = Configurator(
            'canari.resources.templates:dockerize_package',
            project.root_dir,
            {'non_interactive': True},
            variables={'project.name': project.name, 'canari.version': version}
    )

    print('Creating Dockerfile for %s...' % project.name, file=sys.stderr)
    configurator.render()
    print('done!', file=sys.stderr)

    if not find_executable('docker'):
        print("""Could not find 'docker' in your system path. Please download and install Docker from http://docker.com
        and rerun this command again.
        """, file=sys.stderr)
        exit(-1)

    if not args.host:
        if os.name == 'nt':
            args.host = ['']
        elif os.path.exists('/var/run/docker.sock'):
            args.host = ['unix:///var/run/docker.sock']

    docker_hosts = [j for sublist in [('-H', i) for i in args.host] for j in sublist]
    container = '%s/%s:%s' % (project.name, project.name, args.os)

    if not args.host:
        if not find_executable('docker-machine'):
            print("""Could not find 'docker-machine' in your system path. Please download and install Docker Machine from
            http://docker.com and rerun this command again or manually specify a Docker host using the '-H' parameter,
            instead.
            """, file=sys.stderr)
            exit(-1)

        print('Attempting to discover available Docker machines.', file=sys.stderr)
        machines = get_output(run_command(['docker-machine', 'ls', '-q'],stdout=subprocess.PIPE)).split('\n')
        machines.remove('')

        if not machines:
            print('No machines found :(\nExiting...', file=sys.stderr)
            exit(-1)

        machine = question.parse_int('More than one Docker machine was detected. Which one would you like to use to'
                                     'build and run this container?', machines) if len(machines) != 1 else 0

        print('Setting up environment for Docker machine %s' % machines[machine], file=sys.stderr)

        # Inject docker environment variables
        env = get_output(run_command(['docker-machine', 'env', machines[machine]], stdout=subprocess.PIPE))
        os.environ.update(re.findall(r'export ([^=]+)="([^"]+)', env))

    with PushDir(project.root_dir):
        p = run_command(['docker'] + docker_hosts + ['build', '-t', container, '-f', 'Dockerfile-%s' % args.os, '.'])
        p.communicate()
        if p.returncode:
            print('An error occurred while building the Docker container.', file=sys.stderr)
            exit(-1)

    if question.parse_bool('Would you like to run this container now?'):
        port = question.parse_int_range('Which port would you like Plume to listen on externally?', 0, 65535, 8080)
        print('Plume will be listening on http://%s:%s' %
              (re.findall('://([^:]+)', os.environ.get('DOCKER_HOST', 'http://0.0.0.0'))[0], port), file=sys.stderr)
        run_command(['docker'] + docker_hosts + ['run', '-it', '-p', '8080:%s' % port, container]).communicate()

    print('done!', file=sys.stderr)



from __future__ import print_function

import sys

from mrbob.rendering import jinja2_env
from mrbob.configurator import Configurator

from canari.maltego.message import StringEntityField, IntegerEntityField, FloatEntityField, Entity, \
    BooleanEntityField, TimeSpanEntityField, DateTimeEntityField, DateEntityField, LongEntityField
from canari.pkgutils.transform import TransformDistribution
from canari.project import CanariProject
from canari.commands.common import canari_main
from canari.commands.framework import SubCommand, Argument

__author__ = 'Tomas Lima'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = ['Tomas Lima']

__license__ = 'GPLv3'
__version__ = '0.5'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def parse_args(args):
    project = CanariProject(args.out_path)
    args.out_path = project.root_dir
    sys.path.insert(0, project.src_dir)
    return args


@SubCommand(
    canari_main,
    help='Create entities documentation from Canari python classes file.',
    description='Create entities documentation from Canari python classes file.'
)
@Argument(
    '--out-path',
    '-o',
    metavar='<path>',
    help='Where the output entities.rst file will be written to.',
    required=False
)
@Argument(
    'package',
    metavar='<package>',
    help='the name of the canari transforms package to install.'
)
def generate_entities_doc(args):
    args = parse_args(args)

    transform_package = TransformDistribution(args.package)

    jinja2_env.filters['entity_properties'] = \
        lambda v: reversed([(p, getattr(v, p)) for p in dir(v) if isinstance(getattr(v, p), StringEntityField) and
                            not hasattr(Entity, p)])

    def get_property_type(v):
        if isinstance(v, IntegerEntityField):
            return 'int'
        elif isinstance(v, FloatEntityField):
            return 'float'
        elif isinstance(v, BooleanEntityField):
            return 'bool'
        elif isinstance(v, TimeSpanEntityField):
            return 'timedelta'
        elif isinstance(v, DateTimeEntityField):
            return 'datetime'
        elif isinstance(v, DateEntityField):
            return 'date'
        elif isinstance(v, LongEntityField):
            return 'long'
        else:
            return 'str'

    jinja2_env.filters['get_property_type'] = get_property_type

    entity_module = 'canari.maltego.entities' if transform_package.name == 'canari' \
        else '%s.transforms.common.entities' % transform_package.name

    variables = {
        'transform.module': entity_module,
        'transform.entities': transform_package.entities,
        'transform.author': '%s <%s>' % (transform_package.author, transform_package.author_email)
    }

    configurator = Configurator(
            'canari.resources.templates:generate_entities_doc',
            args.out_path,
            {'non_interactive': True},
            variables=variables
    )

    configurator.ask_questions()

    print('Creating entities.rst documentation for %r...' % args.package, file=sys.stderr)
    configurator.render()

    print('done!', file=sys.stderr)



#!/usr/bin/env python

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'banner',
    'common',
    'create_package',
    'create_profile',
    'create_transform',
    'create_aws_lambda',
    'csv2sheets',
    'debug_transform',
    'dockerize_package',
    'framework',
    'generate_entities',
    'help',
    'install_plume',
    'list_transforms',
    'list_commands',
    'load_plume_package',
    'mtgx2csv',
    'run_transform',
    'shell',
    'unload_plume_package',
    'version',
    'generate_entities_doc',
    'remote_transform'
]


#!/usr/bin/env python
from __future__ import print_function
from future.builtins import bytes
from past.builtins import unicode
import sys
import six

# For some reason this function is being used in safedexml. We're going to avoid it completely.
six.u = unicode

from canari.mode import set_canari_mode, CanariMode
from canari.commands.common import canari_main
# Do not remove this line. It loads all our sub-commands into the main commander.
# noinspection PyUnresolvedReferences
from canari.commands import *

import subprocess
import getpass

from canari.utils.fs import FileMutex
from canari.easygui import passwordbox


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.2'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'main',
    'dispatcher',
    'sudo'
]


def main():
    try:
        set_canari_mode(CanariMode.LocalUnknown)
        canari_main()
    except KeyboardInterrupt:
        print('exiting...', file=sys.stderr)


def dispatcher():
    set_canari_mode(CanariMode.LocalDispatch)
    run_transform.run_transform.parser.prog = 'dispatcher'
    opts = run_transform.run_transform.parser.parse_args()
    run_transform.run_transform(opts)


def sudo():
    if not sys.argv[1:]:
        print('usage: %s <command>' % sys.argv[0], file=sys.stderr)
        exit(-1)

    # Let's try and run it right away to see what happens
    p = subprocess.Popen(['sudo', '-S'] + sys.argv[1:], stdin=subprocess.PIPE)
    p.communicate()

    # It ran!
    if not p.returncode:
        exit(0)

    # It didn't :( - let's lock this region now to avoid having multiple password boxes pop-up
    l = FileMutex('pysudo.%s.lock' % getpass.getuser())

    # Try running it again (maybe another process authenticated... why ask for a password again?)
    p = subprocess.Popen(['sudo', '-S'] + sys.argv[1:], stdin=subprocess.PIPE)
    p.communicate()

    if not p.returncode:
        l.unlock()
        exit(0)

    # No we really need to ask for a password :(
    for i in range(0, 3):
        password = passwordbox('Please enter your password.', 'sudo', '')
        if password is None:
            exit(1)

        # Try it out with a password now!
        p = subprocess.Popen(['sudo', '-S', 'true'], stdin=subprocess.PIPE)
        p.communicate(input=bytes('%s\n' % password, 'utf-8'))

        # Did it work? Yes: let's do it!
        if not p.returncode:
            l.unlock()
            p = subprocess.Popen(['sudo', '-S'] + sys.argv[1:], stdin=subprocess.PIPE)
            p.communicate(input=bytes('%s\n' % password, 'utf-8'))
            exit(p.returncode)

    exit(2)



__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'CanariMode',
    'set_canari_mode',
    'get_canari_mode',
    'is_local_exec_mode',
    'is_debug_exec_mode',
    'is_dispatch_exec_mode',
    'is_remote_exec_mode',
    'is_plume_exec_mode',
    'is_lambda_exec_mode',
    'is_shell_exec_mode',
    'is_unknown_exec_mode',
    'is_local_debug_exec_mode',
    'is_local_dispatch_exec_mode',
    'is_local_unknown_exec_mode',
    'is_remote_plume_debug_exec_mode',
    'is_remote_plume_dispatch_exec_mode',
    'is_remote_plume_lambda_debug_exec_mode',
    'is_remote_plume_lambda_dispatch_exec_mode',
    'is_remote_unknown_exec_mode',
    'is_local_shell_debug_exec_mode',
    'get_canari_mode_str'
]


class CanariMode:
    """
    Utility class that provides enumeration variables for Canari operating modes as well as a helper function to convert
    numeric representations of those operating modes into human readable strings. Currently there are two primary
    operating modes, Local and Remote. Each operating mode has three context mode bits:

     - Dispatch: disables debugging and operates in production mode.
     - Debug: enables additional debug messaging.
     - Unknown: disables production and debugging modes - operation is not guaranteed.

    One can also specify which runner is used in Remote mode by adding the Plume and Runner operating modes.
    """

    Local = 0x01
    Remote = 0x02
    Debug = 0x04
    Dispatch = 0x08
    Plume = 0x10
    Shell = 0x40
    Unknown = 0x80
    Lambda = 0x100
    RemotePlumeDispatch = Remote | Dispatch | Plume
    RemotePlumeDebug = Remote | Debug | Plume
    RemotePlumeLambdaDispatch = Remote | Lambda | Plume | Dispatch
    RemotePlumeLambdaDebug = Remote | Lambda | Plume | Debug
    RemoteUnknown = Remote | Unknown
    LocalDispatch = Local | Dispatch
    LocalDebug = Local | Debug
    LocalUnknown = Local | Unknown
    LocalShellDebug = Local | Shell | Debug

    _str_table = {
        Local: 'Local',
        Remote: 'Remote',
        Debug: 'Debug',
        Dispatch: 'Dispatch',
        Plume: 'Plume',
        Shell: 'Shell',
        Unknown: 'Unknown',
        RemotePlumeDispatch: 'Remote (server:Plume, debug:False)',
        RemotePlumeDebug: 'Remote (server:Plume, debug:True)',
        RemoteUnknown: 'Remote (server:?, debug:?)',
        LocalDispatch: 'Local (runner:Canari, debug:False)',
        LocalDebug: 'Local (runner:Canari, debug:True)',
        LocalUnknown: 'Local (runner=?, debug=?)',
        LocalShellDebug: 'Local (runner=shell, debug=True)'
    }

    @classmethod
    def to_str(cls, mode):
        """
        Converts a numeric operating mode into a human readable string.

        :param mode: the numeric operating mode.
        :return: a human readable string that corresponds to the operating mode.
        """
        return cls._str_table.get(mode, cls.Unknown)


# By default the operating mode is unknown.
canari_mode = CanariMode.Unknown


def set_canari_mode(mode=CanariMode.Unknown):
    """
    Sets the global operating mode for Canari. This is used to alter the behaviour of dangerous classes like the
    CanariConfigParser.

    :param mode: the numeric Canari operating mode (CanariMode.Local, CanariMode.Remote, etc.).
    :return: previous operating mode.
    """
    global canari_mode
    old_mode = canari_mode
    canari_mode = mode
    return old_mode


def get_canari_mode():
    """
    Returns the global Canari numeric operating mode.

    :return: global Canari numeric operating mode.
    """
    return canari_mode


def get_canari_mode_str():
    """
    Returns a human readable string of the global Canari operating mode.

    :return: human readable string of the global Canari operating mode.
    """
    return CanariMode.to_str(get_canari_mode())


def is_local_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in local mode.

    :return: True if Canari is operating in local mode.
    """
    return get_canari_mode() & CanariMode.Local


def is_remote_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in remote mode.

    :return: True if Canari is operating in remote mode.
    """
    return get_canari_mode() & CanariMode.Remote


def is_debug_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in debug mode.

    :return: True if Canari is operating in debug mode.
    """
    return get_canari_mode() & CanariMode.Debug


def is_dispatch_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in dispatch mode.

    :return: True if Canari is operating in dispatch mode.
    """
    return get_canari_mode() & CanariMode.Dispatch


def is_plume_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in Plume mode.

    :return: True if Canari is operating in Plume mode.
    """
    return get_canari_mode() & CanariMode.Plume


def is_lambda_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in Plume mode.

    :return: True if Canari is operating in Plume mode.
    """
    return get_canari_mode() & CanariMode.Lambda


def is_unknown_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in an unknown mode.

    :return: True if Canari is operating in an unknown mode.
    """
    return get_canari_mode() & CanariMode.Unknown


def is_remote_plume_dispatch_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in Plume dispatch mode.

    :return: True if Canari is operating in Plume dispatch mode.
    """
    return is_plume_exec_mode() and is_dispatch_exec_mode()


def is_remote_plume_debug_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in Plume debug mode.

    :return: True if Canari is operating in Plume debug mode.
    """
    return is_plume_exec_mode() and is_debug_exec_mode()


def is_remote_plume_lambda_dispatch_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in Plume dispatch mode.

    :return: True if Canari is operating in Plume dispatch mode.
    """
    return is_plume_exec_mode() and is_dispatch_exec_mode() and is_lambda_exec_mode()


def is_remote_plume_lambda_debug_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in Plume debug mode.

    :return: True if Canari is operating in Plume debug mode.
    """
    return is_plume_exec_mode() and is_debug_exec_mode() and is_lambda_exec_mode()


def is_remote_unknown_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in remote Runner debug mode.

    :return: True if Canari is operating in remote Runner mode.
    """
    return is_remote_exec_mode() and is_unknown_exec_mode()


def is_local_dispatch_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in local dispatch mode.

    :return: True if Canari is operating in local dispatch mode.
    """
    return is_local_exec_mode() and is_dispatch_exec_mode()


def is_local_debug_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in local debug mode.

    :return: True if Canari is operating in local debug mode.
    """
    return is_local_exec_mode() and is_debug_exec_mode()


def is_local_unknown_exec_mode():
    """
    Returns a boolean specifying whether Canari is operating in local unknown mode.

    :return: True if Canari is operating in local unknown mode.
    """
    return is_local_exec_mode() and is_unknown_exec_mode()


def is_shell_exec_mode():
    return get_canari_mode() & CanariMode.Shell


def is_local_shell_debug_exec_mode():
    return is_local_exec_mode() and is_shell_exec_mode() and is_debug_exec_mode()


from past.builtins import basestring

from subprocess import Popen
import os
import re

from canari.mode import is_remote_exec_mode
from canari.resource import external_resource
from canari.utils.stack import calling_package

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'RequireSuperUser',
    'Deprecated',
    'EnableRemoteExecution',
    'EnableDebugWindow',
    'ExternalCommand'
]


def RequireSuperUser(transform):
    """
    Marks the transform as a privileged transform which requires root access to execute. Upon transform execution, a
    sudo login box will appear prompting the user to enter their sudo password in the event that there isn't a pre-
    existing sudo session.

    .. note::
    This is not compatible with Windows. UAC must be turned off for privileged functions to execute.

    :param transform: the transform class that will be marked as privileged.
    :return: the superuser'ed transform class.

    :Example:

    @RequireSuperUser
    class MyTransform(Transform):
        pass
    """
    transform.superuser = True
    return transform


def Deprecated(transform):
    """
    Marks the transform as deprecated.

    :Example:

    @Deprecated
    class MyTransform(Transform):
        pass

    :param transform: the transform class that will be marked as deprecated.
    :return: the deprecated transform class.
    """
    transform.deprecated = True
    return transform


def EnableRemoteExecution(transform):
    """
    Marks the transform as a remote transform. This allows Plume to import and execute the transform.

    :Example:

    @EnableRemoteExecution
    class MyTransform(Transform):
        pass

    :param transform: the transform class that will be marked as deprecated.
    :return: the deprecated transform class.
    """
    transform.remote = True
    return transform


class RequestFilter(object):

    def __init__(self, filter_, remote_only=False, **kwargs):
        self.filter = filter_
        self.remote_only = remote_only

    def __call__(self, transform):
        if callable(filter):
            if self.remote_only and is_remote_exec_mode():
                orig_do_transform = transform.do_transform

                def do_transform(self_, request, response, config):
                    if self.filter.__call__(request, response, config):
                        return response
                    return orig_do_transform(self_, request, response, config)

                transform.do_transform = do_transform
            return transform
        raise ValueError('Expected callable (got %s instead).' % type(self.filter).__name__)


def EnableDebugWindow(transform):
    """
    TODO.

    :Example:

    @EnableDebugWindow
    MyTransform(Transform):
        pass

    :param transform: the transform class that will be marked as deprecated.
    :return: the deprecated transform class.
    """
    transform.debug = True
    return transform


class ExternalCommand(object):
    def __init__(self, interpreter, program, args=None):
        if args is None:
            args = []
        self.args = []

        if interpreter:
            self.args.append(interpreter)
            libpath = external_resource(
                os.path.dirname(program),
                '%s.resources.external' % calling_package()
            )
            if interpreter.startswith('perl') or interpreter.startswith('ruby'):
                self.args.append('-I%s' % libpath)
            elif interpreter.startswith('java'):
                self.args.extend(['-cp', libpath])

        if ' ' in program:
            raise ValueError('Transform name %r cannot have spaces.' % program)
        else:
            self.args.append(
                external_resource(
                    program,
                    '%s.resources.external' % calling_package()
                )
            )

        if isinstance(args, basestring):
            self.args = re.split(r'\s+', args)
        else:
            self.args.extend(args)

    def __call__(self, request, *args):
        self.args.append(request.entity.value)
        if isinstance(request.parameters, list) and request.parameters:
            self.args.extend(request.parameters)
        if request.entity.fields:
            self.args.append(
                '#'.join(
                    ['%s=%s' % (k, v.value.replace('#', '\\#').replace('=', '\\='))
                     for k, v in request.entity.fields.items()]
                )
            )
        p = Popen(self.args)
        p.communicate()
        exit(p.returncode)



from __future__ import print_function

import os
import re
import sys
import unicodedata
from getpass import getuser
from argparse import Action

from past.builtins import basestring, unicode

from canari.commands.framework import Command
from canari.config import load_config, OPTION_LOCAL_PATH

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


class ParseFieldsAction(Action):
    """
    Custom argparse action to parse arguments for the run- and debug-transform commands. This ensures that all
    positional arguments are parsed and stored correctly.
    """

    def __call__(self, parser, namespace, values, option_string=None):
        # Does the value argument have an equals ('=') sign that is not escaped and is the params argument populated?
        if namespace.params and re.search(r'(?<=[^\\])=', namespace.value):
            # if so, apply fix and pop last element of namespace.params into namespace.value
            # and copy what was in namespace.value into namespace.fields to fix everything
            values = namespace.value
            namespace.value = namespace.params.pop().replace('\=', '=')
            namespace.fields = values
            # Next parse our fields argument into a dictionary
            fields = re.split(r'(?<=[^\\])#', values)
            if fields:
                namespace.fields = dict(
                    map(
                        lambda x: [
                            field.replace('\#', '#').replace('\=', '=').replace('\\\\', '\\')
                            for field in re.split(r'(?<=[^\\])=', x, 1)
                        ],
                        fields
                    )
                )


@Command(description='Centralized Canari Management System')
def canari_main(opts):
    """
    This is the main function for the Canari commander. Nothing special here.
    """
    profile_dir = os.path.join(os.path.expanduser('~'), '.canari')
    if not os.path.lexists(profile_dir):
        os.makedirs(profile_dir)
    fix_pypath()
    fix_binpath(load_config()[OPTION_LOCAL_PATH])
    if not hasattr(opts, 'command_function'):
        canari_main.parser.print_help()
    else:
        opts.command_function(opts)


def to_utf8(s):
    return unicodedata.normalize('NFKD', unicode(s)).encode('ascii', 'ignore')


def uproot():
    if os.name == 'posix' and not os.geteuid():
        login = getuser()

        if login != 'root':
            import pwd
            print('Why are you using root to run this command? You should be using %s! Bringing you down...' % login,
                  file=sys.stderr)
            user = pwd.getpwnam(login)
            os.setgid(user.pw_gid)
            os.setuid(user.pw_uid)


def fix_pypath():
    if '' not in sys.path:
        sys.path.insert(0, '')


def fix_binpath(paths):
    if paths is not None and paths:
        if isinstance(paths, basestring):
            os.environ['PATH'] = paths
        elif isinstance(paths, list):
            os.environ['PATH'] = os.pathsep.join(paths)





from canari.maltego.message import (Entity, StringEntityField, IntegerEntityField,
                                    FloatEntityField, BooleanEntityField, DateEntityField)

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'Affiliation',
    'Bebo',
    'Facebook',
    'Flickr',
    'Linkedin',
    'MySpace',
    'Orkut',
    'Spock',
    'Twitter',
    'WikiEdit',
    'Zoominfo',
    'Alias',
    'AS',
    'Banner',
    'BuiltWithTechnology',
    'Device',
    'DNSName',
    'Document',
    'Domain',
    'EmailAddress',
    'FacebookObject',
    'File',
    'GPS',
    'Image',
    'IPv4Address',
    'Location',
    'MXRecord',
    'Netblock',
    'NominatimLocation',
    'NSRecord',
    'Person',
    'PhoneNumber',
    'Phrase',
    'Port',
    'Service',
    'Twit',
    'URL',
    'Vulnerability',
    'Webdir',
    'Website',
    'WebTitle',
    'TrackingCode',
    'CircularArea',
    'Hash',
    'Hashtag'
]


class Unknown(Entity):
    _category_ = 'Unknown'


class GPS(Entity):
    _category_ = 'Locations'
    gps = StringEntityField('properties.gps', display_name='GPS Co-ordinate', is_value=True)
    latitude = FloatEntityField('latitude', display_name='Latitude')
    longitude = FloatEntityField('longitude', display_name='Longitude')


class Device(Entity):
    _category_ = 'Devices'
    device = StringEntityField('properties.device', display_name='Device')


class BuiltWithTechnology(Entity):
    _category_ = 'Penetration Testing'
    builtwith = StringEntityField('properties.builtwithtechnology', display_name='BuiltWith Technology')


class Domain(Entity):
    _category_ = 'Infrastructure'
    fqdn = StringEntityField('fqdn', display_name='Domain Name', is_value=True)
    whois_info = StringEntityField('whois-info', display_name='WHOIS Info', alias='whois')


class DNSName(Entity):
    _category_ = 'Infrastructure'
    fqdn = StringEntityField('fqdn', display_name='DNS Name', is_value=True)


class MXRecord(DNSName):
    priority = IntegerEntityField('mxrecord.priority', display_name='Priority')


class NSRecord(DNSName):
    pass


class IPv4Address(Entity):
    _category_ = 'Infrastructure'
    _alias_ = 'IPAddress'
    ipv4address = StringEntityField('ipv4-address', display_name='IP Address', is_value=True)
    internal = BooleanEntityField('ipaddress.internal', display_name='Internal')


class Netblock(Entity):
    _category_ = 'Infrastructure'
    ipv4range = StringEntityField('ipv4-range', display_name='IP Range', is_value=True)


class AS(Entity):
    _category_ = 'Infrastructure'
    _alias_ = 'ASNumber'
    number = IntegerEntityField('as.number', display_name='AS Number', is_value=True)


class Website(Entity):
    _category_ = 'Infrastructure'
    fqdn = StringEntityField('fqdn', display_name='Website', is_value=True)
    ssl_enabled = BooleanEntityField('website.ssl-enabled', display_name='SSL Enabled')
    ports = IntegerEntityField('ports', display_name='Ports')


class URL(Entity):
    _category_ = 'Infrastructure'
    short_title = StringEntityField('short-title', display_name='Short title', is_value=True,
                                    alias='maltego.v2.value.property')
    url = StringEntityField('url', display_name='URL', alias='theurl')
    title = StringEntityField('title', display_name='Title', alias='fulltitle')


class Phrase(Entity):
    _category_ = 'Personal'
    text = StringEntityField('text', display_name='Text', is_value=True)


class Document(Entity):
    _category_ = 'Personal'
    url = StringEntityField('url', display_name='URL', alias='link', is_value=True)
    title = StringEntityField('title', display_name='Title', alias='maltego.v2.value.property')
    metadata = StringEntityField('document.metadata', display_name='Meta-Data', alias='metainfo')


class Person(Entity):
    _category_ = 'Personal'
    fullname = StringEntityField('person.fullname', display_name='Full Name', is_value=True)
    lastname = StringEntityField('person.lastname', display_name='Surname', alias='lastname')
    firstnames = StringEntityField('person.firstnames', display_name='First Names', alias='firstname')


class EmailAddress(Entity):
    _category_ = 'Personal'
    email = StringEntityField('email', display_name='Email Address', is_value=True)


class Twit(Entity):
    _category_ = 'Social Network'
    name = StringEntityField('twit.name', display_name='Twit', is_value=True)
    content = StringEntityField('content', display_name='Content')
    pubdate = StringEntityField('pubdate', display_name='Date published')
    img_link = StringEntityField('img_link', display_name='Image Link', alias='imglink')
    author = StringEntityField('author', display_name='Author')
    title = StringEntityField('title', display_name='Title')
    author_uri = StringEntityField('author_uri', display_name='Author URI')
    id = StringEntityField('id', display_name='Twit ID')


class Affiliation(Entity):
    _category_ = 'Social Network'
    _namespace_ = 'maltego.affiliation'
    person_name = StringEntityField('person.name', display_name='Name', is_value=True)
    uid = StringEntityField('affiliation.uid', display_name='UID', alias='uid')
    network = StringEntityField('affiliation.network', display_name='Network', alias='network')
    profile_url = StringEntityField('affiliation.profile-url', display_name='Profile URL', alias='profile_url')


class Bebo(Affiliation):
    _alias_ = 'AffiliationBebo'


class Facebook(Affiliation):
    _alias_ = 'AffiliationFacebook'


class Flickr(Affiliation):
    _alias_ = 'AffiliationFlickr'


class Linkedin(Affiliation):
    _alias_ = 'AffiliationLinkedin'


class MySpace(Affiliation):
    _alias_ = 'AffiliationMySpace'


class Orkut(Affiliation):
    _alias_ = 'AffiliationOrkut'


class Twitter(Affiliation):
    _alias_ = 'AffiliationTwitter'
    number = IntegerEntityField('twitter.number', display_name='Twitter Number')
    screenname = StringEntityField('twitter.screen-name', display_name='Screen Name')
    friendcount = IntegerEntityField('twitter.friendcount', display_name='Friend Count')
    fullname = StringEntityField('person.fullname', display_name='Real Name')


class Zoominfo(Affiliation):
    pass


class WikiEdit(Affiliation):
    pass


class Spock(Affiliation):
    _alias_ = 'AffiliationSpock'
    websites = StringEntityField('spock.websites', display_name='Listed Websites')


class FacebookObject(Entity):
    _category_ = 'Social Network'
    object = StringEntityField('properties.facebookobject', display_name='Facebook Object')


class Location(Entity):
    _category_ = 'Locations'
    name = StringEntityField('location.name', display_name='Name', is_value=True)
    city = StringEntityField('city', display_name='City')
    countrycode = StringEntityField('countrycode', display_name='Country Code', alias='countrysc')
    area = StringEntityField('location.area', display_name='Area', alias='area')
    country = StringEntityField('country', display_name='Country')
    longitude = FloatEntityField('longitude', display_name='Longitude', alias='long')
    latitude = FloatEntityField('latitude', display_name='Latitude', alias='lat')
    streetaddress = StringEntityField('streetaddress', display_name='Street Address')
    areacode = StringEntityField('location.areacode', display_name='Area Code')


class NominatimLocation(Entity):
    _category_ = 'Locations'
    nominatim = StringEntityField('properties.nominatimlocation', display_name='Nominatim Location', is_value=True)


class PhoneNumber(Entity):
    _category_ = 'Personal'
    phonenumber = StringEntityField('phonenumber', display_name='Phone Number', is_value=True)
    areacode = StringEntityField('phonenumber.areacode', display_name='Area Code', alias='areacode')
    lastnumbers = StringEntityField('phonenumber.lastnumbers', display_name='Last Digits', alias='lastnumbers')
    citycode = StringEntityField('phonenumber.citycode', display_name='City Code', alias='citycode')
    countrycode = StringEntityField('phonenumber.countrycode', display_name='Country Code', alias='countrycode')


class Alias(Entity):
    _category_ = 'Personal'
    alias = StringEntityField('properties.alias', display_name='Alias')


class File(Entity):
    _category_ = 'Personal'
    source = StringEntityField('source', display_name='Source')
    description = StringEntityField('description', display_name='Description')


class Image(Entity):
    _category_ = 'Personal'
    description = StringEntityField('properties.image', display_name='Description')
    url = StringEntityField('fullImage', display_name='URL')


class Banner(Entity):
    _category_ = 'Infrastructure'
    text = StringEntityField('banner.text', display_name='Banner', is_value=True)


class Port(Entity):
    _category_ = 'Infrastructure'
    number = StringEntityField('port.number', display_name='Ports', is_value=True)


class Service(Entity):
    _category_ = 'Infrastructure'
    name = StringEntityField('service.name', display_name='Description', is_value=True)
    banner = StringEntityField('banner.text', display_name='Service Banner')
    ports = StringEntityField('port.number', display_name='Ports')


class Vulnerability(Entity):
    _category_ = 'Penetration Testing'
    _alias_ = 'Vuln'
    id = StringEntityField('vulnerability.id', display_name='ID', is_value=True)


class Webdir(Entity):
    _category_ = 'Infrastructure'
    name = StringEntityField('directory.name', display_name='Name', is_value=True)


class WebTitle(Entity):
    _category_ = 'Infrastructure'
    title = StringEntityField('title', display_name='Title', is_value=True)


class TrackingCode(Entity):
    _alias_ = 'maltego.UniqueIdentifier'
    _category_ = 'Infrastructure'
    unique_identifier = StringEntityField('properties.uniqueidentifier', display_name='Uniqueidentifier', is_value=True)
    identifier_type = StringEntityField('identifierType', display_name='Identifier Type')


class CircularArea(Entity):
    _category_ = 'Locations'
    area_circular = StringEntityField('area.circular', display_name='Circular Area', is_value=True)
    latitude = FloatEntityField('latitude', display_name='Latitude')
    longitude = FloatEntityField('longitude', display_name='Longitude')
    radius = IntegerEntityField('radius', display_name='Radius (m)')


class Hash(Entity):
    _category_ = 'Malware'
    hash = StringEntityField('properties.hash', display_name='Hash', is_value=True)
    type = StringEntityField('type', display_name='Hash Type')
    owner = StringEntityField('owner', display_name='Owner')
    before = DateEntityField('before', display_name='Before')
    after = DateEntityField('after', display_name='After')
    included_media_types = StringEntityField('includeMediaType', display_name='Included Media Types')
    excluded_media_types = StringEntityField('excludeMediaType', display_name='Excluded Media Types')


class Hashtag(Entity):
    _category_ = 'Social'
    hashtag = StringEntityField('twitter.hashtag', display_name='Hashtag', is_value=True)



from imghdr import what
from os.path import abspath

from pkg_resources import resource_filename, resource_listdir, resource_isdir

from canari.utils.stack import calling_package

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'external_resource',
    'image_resources',
    'icon_resource',
    'global_config'
]


def external_resource(name, package=None):
    """
    Returns the absolute path to the external resource requested. If a package is not explicitly specified then the
    calling package name is used.

    :param name: path relative to package path of the external resource.
    :param package: package name in dotted format.
    :return: the absolute path to the external resource.
    """
    if not package:
        package = '%s.resources.external' % calling_package()
    return resource_filename(package, name)


def icon_resource(name, package=None):
    """
    Returns the absolute URI path to an image. If a package is not explicitly specified then the calling package name is
    used.

    :param name: path relative to package path of the image resource.
    :param package: package name in dotted format.
    :return: the file URI path to the image resource (i.e. file:///foo/bar/image.png).
    """
    if not package:
        package = '%s.resources.images' % calling_package()
    name = resource_filename(package, name)
    if not name.startswith('/'):
        return 'file://%s' % abspath(name)
    return 'file://%s' % name


def image_resources(package=None, directory='resources'):
    """
    Returns all images under the directory relative to a package path. If no directory or package is specified then the
    resources module of the calling package will be used. Images are recursively discovered.

    :param package: package name in dotted format.
    :param directory: path relative to package path of the resources directory.
    :return: a list of images under the specified resources path.
    """
    if not package:
        package = calling_package()
    package_dir = '.'.join([package, directory])
    images = []
    for i in resource_listdir(package, directory):
        if i.startswith('__') or i.endswith('.egg-info'):
            continue
        fname = resource_filename(package_dir, i)
        if resource_isdir(package_dir, i):
            images.extend(image_resources(package_dir, i))
        elif what(fname):
            images.append(fname)
    return images


# etc
global_config = resource_filename('canari.resources.etc', 'canari.conf')



from past.builtins import raw_input

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2016, canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = []


def parse_bool(question, default=True):
    choices = 'Y/n' if default else 'y/N'
    default = 'Y' if default else 'N'
    while True:
        ans = raw_input('%s [%s]: ' % (question, choices)).upper() or default
        if ans.startswith('Y'):
            return True
        elif ans.startswith('N'):
            return False
        else:
            print('Invalid selection (%s) must be either [y]es or [n]o.' % ans)


def parse_int(question, choices, default=0):
    while True:
        for i, c in enumerate(choices):
            print('[%d] - %s' % (i, c))
        ans = raw_input('%s [%d]: ' % (question, default)) or default
        try:
            ans = int(ans)
            if not 0 <= ans <= i:
                raise ValueError
            return ans
        except ValueError:
            print('Invalid selection (%s) must be an integer between 0 and %d.' % (ans, i))


def parse_int_range(question, from_, to, default=0):
    from_ = int(from_)
    to = int(to)

    if from_ > to:
        temp = from_
        from_ = to
        to = temp

    while True:
        ans = parse_str('%s. Valid values are between %d and %d' % (question, from_, to), default)
        try:
            ans = int(ans)
            if not from_ <= ans <= to:
                raise ValueError
            return ans
        except ValueError:
            print('Invalid selection (%s) must be an integer between %d and %d.' % (ans, from_, to))


def parse_str(question, default):
    return raw_input('%s [%s]: ' % (question, default)) or default


"""
This module defines the various configuration elements that appear in the Maltego profile files (*.mtz). These
configuration elements specify the configuration options for Maltego transforms, servers, entities, machines, and
viewlets. Canari uses these elements to generate Maltego profiles that can be imported into Maltego.
"""

import time
from canari.maltego.oxml import MaltegoElement, fields as fields_


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'TransformAdapter',
    'VisibilityType',
    'BuiltInTransformSets',
    'Set',
    'Transform',
    'TransformSet',
    'InputConstraint',
    'OutputEntity',
    'InputEntity',
    'PropertyType',
    'TransformProperty',
    'TransformPropertySetting',
    'CmdLineTransformProperty',
    'CmdLineTransformPropertySetting',
    'CmdParmTransformProperty',
    'CmdParmTransformPropertySetting',
    'CmdCwdTransformProperty',
    'CmdCwdTransformPropertySetting',
    'CmdDbgTransformProperty',
    'CmdDbgTransformPropertySetting',
    'TransformSettings',
    'Protocol',
    'AuthenticationType',
    'Authentication',
    'MaltegoServer',
    'EntityCategory',
    'attr',
    'fileobject',
    'attributes',
    'Properties',
    'MaltegoTransform'
]


class TransformAdapter:
    """Defines the transform adapter for the running transform. Currently, there are only two transform adapters
    available in Maltego. They are listed below. This is just an enumeration class.
    """
    Local = 'com.paterva.maltego.transform.protocol.v2.LocalTransformAdapterV2'
    Localv2 = 'com.paterva.maltego.transform.protocol.v2api.LocalTransformAdapterV2'
    Remote = 'com.paterva.maltego.transform.protocol.v2.RemoteTransformAdapterV2'


class VisibilityType:
    """An enumeration class that defines the visibility of a transform."""
    Public = 'public'
    Private = 'private'


class BuiltInTransformSets:
    """An enumeration class that specifies the names of built-in transform sets in Maltego"""
    ConvertToDomain = "Convert to Domain"
    DomainsUsingMXNS = "Domains using MX NS"
    FindOnWebpage = "Find on webpage"
    RelatedEmailAddresses = "Related Email addresses"
    DNSFromDomain = "DNS from Domain"
    EmailAddressesFromDomain = "Email addresses from Domain"
    IPOwnerDetail = "IP owner detail"
    ResolveToIP = "Resolve to IP"
    DNSFromIP = "DNS from IP"
    EmailAddressesFromPerson = "Email addresses from Person"
    InfoFromNS = "Info from NS"
    DomainFromDNS = "Domain From DNS"
    FilesAndDocumentsFromDomain = "Files and Documents from Domain"
    LinksInAndOutOfSite = "Links in and out of site"
    DomainOwnerDetail = "Domain owner detail"
    FilesAndDocumentsFromPhrase = "Files and Documents from Phrase"


class Set(MaltegoElement):
    name = fields_.String()


class Transform(MaltegoElement):
    name = fields_.String()


class TransformSet(MaltegoElement):

    name = fields_.String()
    description = fields_.String(default='')
    transforms = fields_.List(Transform, tagname='Transforms')

    def __iadd__(self, other):
        if isinstance(other, Transform):
            self.transforms.append(other)
        return self


class InputConstraint(MaltegoElement):

    class meta:
        tagname = 'Entity'

    type = fields_.String()
    min = fields_.Integer(default=1)
    max = fields_.Integer(default=1)


class OutputEntity(InputConstraint):

    class meta:
        tagname = 'Entity'


class InputEntity(InputConstraint):

    class meta:
        tagname = 'Entity'


class PropertyType(object):
    String = 'string'
    Boolean = 'boolean'
    Integer = 'int'


class TransformProperty(MaltegoElement):

    class meta:
        tagname = 'Property'

    defaultvalue = fields_.String(tagname='DefaultValue', required=False)
    samplevalue = fields_.String(tagname='SampleValue', default='')
    abstract = fields_.Boolean(default=False)
    description = fields_.String(default='')
    displayname = fields_.String(attrname='displayName', required=False)
    hidden = fields_.Boolean(default=False)
    name = fields_.String()
    nullable = fields_.Boolean(default=False)
    readonly = fields_.Boolean(default=False)
    popup = fields_.Boolean(default=False)
    type = fields_.String(default=PropertyType.String)
    visibility = fields_.String(default=VisibilityType.Public)


class TransformPropertySetting(MaltegoElement):

    class meta:
        tagname='Property'

    name = fields_.String()
    popup = fields_.Boolean(default=False)
    type = fields_.String(default=PropertyType.String)
    value = fields_.String(tagname=".")


def CmdLineTransformProperty(cmd=''):
    return TransformProperty(
        name='transform.local.command',
        defaultvalue=cmd,
        displayname='Command line',
        description='The command to execute for this transform'
    )


def CmdLineTransformPropertySetting(cmd=''):
    return TransformPropertySetting(
        name='transform.local.command',
        value=cmd
    )


def CmdParmTransformProperty(params=''):
    return TransformProperty(
        name='transform.local.parameters',
        defaultvalue=params,
        displayname='Command parameters',
        description='The parameters to pass to the transform command'
    )


def CmdParmTransformPropertySetting(params=''):
    return TransformPropertySetting(
        name='transform.local.parameters',
        value=params
    )


def CmdCwdTransformProperty(cwd=''):
    return TransformProperty(
        name='transform.local.working-directory',
        defaultvalue=cwd,
        displayname='Working directory',
        description='The working directory used when invoking the executable',
        samplevalue='/'
    )


def CmdCwdTransformPropertySetting(cwd=''):
    return TransformPropertySetting(
        name='transform.local.working-directory',
        value=cwd
    )


def CmdDbgTransformProperty(dbg=False):
    return TransformProperty(
        name='transform.local.debug',
        defaultvalue=str(dbg).lower(),
        displayname='Show debug info',
        description="When this is set, the transform's text output will be printed to the output window",
        samplevalue='false',
        type=PropertyType.Boolean
    )


def CmdDbgTransformPropertySetting(dbg=False):
    return TransformPropertySetting(
        name='transform.local.debug',
        value=str(dbg).lower(),
        type=PropertyType.Boolean
    )


class TransformSettings(MaltegoElement):

    enabled = fields_.Boolean(default=True)
    runwithall = fields_.Boolean(default=True)
    favorite = fields_.Boolean(default=False)
    accepted = fields_.Boolean(default=False, attrname='disclaimerAccepted')
    showhelp = fields_.Boolean(default=True, attrname='showHelp')
    properties = fields_.List(TransformPropertySetting, tagname='Properties')

    def __iadd__(self, other):
        if isinstance(other, TransformPropertySetting):
            self.properties.append(other)
        return self


class Protocol(MaltegoElement):

    version = fields_.String(default='2.0')


class AuthenticationType:
    """An enumeration class that defines the authentication type for a transform. This appears to apply only to Paterva
    transforms."""
    Anonymous = 'none'
    Mac = 'mac'
    License = 'license'


class Authentication(MaltegoElement):

    type = fields_.String(default=AuthenticationType.Anonymous)


class MaltegoServer(MaltegoElement):

    name = fields_.String(default='Local')
    enabled = fields_.Boolean(default=True)
    description = fields_.String(default='Local transforms hosted on this machine')
    url = fields_.String(default='http://localhost')
    lastsync = fields_.String(tagname='LastSync', default=time.strftime('%Y-%m-%d'))
    protocol = fields_.Model(Protocol)
    authentication = fields_.Model(Authentication)
    transforms = fields_.List(Transform, tagname='Transforms')

    def __iadd__(self, other):
        if isinstance(other, Transform):
            self.transforms.append(other)
        return self


class EntityCategory(MaltegoElement):

    name = fields_.String()


class attr(MaltegoElement):
    name = fields_.String()
    stringvalue = fields_.String(required=False)
    boolvalue = fields_.Boolean(required=False)


class fileobject(MaltegoElement):

    name = fields_.String()
    attrs = fields_.Dict(attr, key='name')

    def __iadd__(self, other):
        if isinstance(other, attr):
            self.attrs[other.name] = other
        return self


class attributes(MaltegoElement):

    version = fields_.String(default='1.0')
    fileobjects = fields_.Dict(fileobject, key='name')

    def __iadd__(self, other):
        if isinstance(other, fileobject):
            self.fileobjects[other.name] = other
        return self


class Properties(MaltegoElement):

    fields = fields_.Dict(TransformProperty, key='name', tagname='Fields')

    def __iadd__(self, other):
        if isinstance(other, TransformProperty):
            self.fields[other.name] = other
        return self


class MaltegoTransform(MaltegoElement):
    """This is the complete MaltegoTransform element definition that is present in Maltego profiles. It defines every
    aspect of a transform including it's input entity type, transform set, template, etc."""

    name = fields_.String()
    displayname = fields_.String(attrname='displayName', default='')
    abstract = fields_.String(default=False)
    template = fields_.Boolean(default=False)
    visibility = fields_.String(default=VisibilityType.Public)
    description = fields_.String(default='')
    helpurl = fields_.String(attrname='helpURL', default='')
    author = fields_.String(default='')
    owner = fields_.String(default='')
    locrel = fields_.String(attrname='locationRelevance', default='global')
    version = fields_.String(default='1.0')
    requireinfo = fields_.Boolean(default=False, attrname='requireDisplayInfo')
    adapter = fields_.String(tagname='TransformAdapter', default=TransformAdapter.Local)
    properties = fields_.Model(Properties)
    input = fields_.List(InputConstraint, tagname='InputConstraints', required=False)
    output = fields_.List(OutputEntity, tagname='OutputEntities', required=False)
    help = fields_.CDATA(tagname='Help', default='')
    disclaimer = fields_.CDATA(tagname='Disclaimer', default='')
    sets = fields_.List(Set, tagname='defaultSets')
    stealthlevel = fields_.Integer(tagname='StealthLevel', default=0)
    authenticator = fields_.String(tagname='Authenticator', required=False)

    def __iadd__(self, other):
        if isinstance(other, Set):
            self.sets.append(other)
        elif isinstance(other, TransformProperty):
            self.properties.fields_.append(other)
        elif isinstance(other, InputConstraint) or isinstance(other, InputEntity):
            self.input.append(other)
        elif isinstance(other, OutputEntity):
            self.output.append(other)
        return self


class Field(MaltegoElement):

    name = fields_.String()
    type = fields_.String()
    nullable = fields_.Boolean(default=True)
    hidden = fields_.Boolean(default=False)
    readonly = fields_.Boolean(default=False)
    description = fields_.String(required=False)
    displayname = fields_.String(attrname='displayName', required=False)
    defaultvalue = fields_.String(tagname='DefaultValue', required=False)
    samplevalue = fields_.String(tagname='SampleValue', default='')


class Groups(MaltegoElement):
    pass


class EntityProperties(MaltegoElement):

    class meta:
        tagname = 'Properties'

    value = fields_.String(required=False)
    groups = fields_.Model(Groups, required=False)
    fields = fields_.Dict(Field, key='name', tagname='Fields', required=False)


class RegexGroup(MaltegoElement):

    property = fields_.String(required=False)


class Converter(MaltegoElement):

    value = fields_.CDATA(default='', tagname='Value', required=False)
    regexgroups = fields_.Dict(RegexGroup, key='property', required=False)


class MaltegoEntity(MaltegoElement):

    id = fields_.String()
    displayname = fields_.String(attrname='displayName', required=False)
    plural = fields_.String(attrname='displayNamePlural', required=False)
    description = fields_.String(default='', required=False)
    category = fields_.String(required=False)
    smallicon = fields_.String(attrname='smallIconResource', required=False)
    largeicon = fields_.String(attrname='largeIconResource', required=False)
    allowedRoot = fields_.Boolean(default=True, required=False)
    conversion_order = fields_.Integer(attrname='conversionOrder', default=2147483647, required=False)
    visible = fields_.Boolean(default=True, required=False)
    largeicontag = fields_.String(tagname='Icon', required=False)
    smallicontag = fields_.String(tagname='SmallIcon', required=False)
    converter = fields_.Model(Converter, required=False)
    properties = fields_.Model(EntityProperties, required=False)



from __future__ import print_function

import sys

from canari.commands.common import canari_main
from canari.commands.framework import SubCommand, Argument
from canari.maltego.message import _Entity, Field, Limits, MaltegoMessage
from canari.maltego.runner import remote_canari_transform_runner, console_writer
from canari.mode import set_canari_mode, CanariMode

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2016, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = []


def split_validate(value, type_):
    if '=' not in value:
        print('Invalid %s ("%s") specified. Must be in "name=value" format.' % (type_, value), file=sys.stderr)
        exit(-1)
    return value.split('=', 1)


@SubCommand(
    canari_main,
    help='Runs Canari local transforms in a terminal-friendly fashion.',
    description='Runs Canari local transforms in a terminal-friendly fashion.'
)
@Argument(
    'host',
    metavar='<host[:port]>',
    help='The hostname or IP address and optionally the port of the remote Canari transform server.'
)
@Argument(
    'transform',
    metavar='<transform>',
    help='The UUID of the transform you wish to run (e.g. sploitego3.IPToLocation).'
)
@Argument(
    'input',
    metavar='<entity name>=<value>',
    help='The name and value of the input entity being passed into the local transform (e.g. "maltego.Person=Bob").'
)
@Argument(
    '-f',
    '--entity-field',
    metavar='<name>=<value>',
    # nargs='*',
    help='The entity field name and value pair (e.g. "person.firstname=Bob"). Can be specified multiple times.',
    action='append',
    default=[]
)
@Argument(
    '-p',
    '--transform-parameter',
    metavar='<name>=<value>',
    # nargs='*',
    help='Transform parameter name and value pair (e.g. "api.key=123"). Can be specified multiple times.',
    action='append',
    default=[]
)
@Argument(
    '-r',
    '--raw-output',
    help='Print out raw XML output instead of prettified format.',
    action='store_true',
    default=False
)
@Argument(
    '--ssl',
    help='Perform request over HTTPS (default: False).',
    action='store_true',
    default=False
)
@Argument(
    '-b',
    '--base-path',
    metavar='<base path>',
    default='/',
    help='The base path of the Canari transform server (default: "/").'
)
@Argument(
    '--soft-limit',
    type=int,
    default=500,
    metavar='<soft limit>',
    help='Set the soft limit (default: 500)'
)
@Argument(
    '--hard-limit',
    type=int,
    metavar='<hard limit>',
    default=10000,
    help='Set the hard limit (default: 10000)'
)
@Argument(
    '-v',
    '--verbose',
    help='Enable verbose debug mode.',
    action='store_true',
    default=False
)
def remote_transform(args):
    if args.verbose:
        set_canari_mode(CanariMode.LocalDebug)

    entity_type, entity_value = split_validate(args.input, 'entity')
    fields = {}
    params = []

    for f in args.entity_field:
        name, value = split_validate(f, 'entity field')
        fields[name] = Field(name=name, value=value)

    for p in args.transform_parameter:
        name, value = split_validate(p, 'transform parameter')
        params.append(Field(name=name, value=value))

    try:
        r = remote_canari_transform_runner(
            args.host,
            args.base_path,
            args.transform,
            [_Entity(type=entity_type, value=entity_value, fields=fields)],
            params,
            Limits(soft=args.soft_limit, hard=args.hard_limit),
            args.ssl
        )

        if r.status == 200:
            data = r.read()
            if args.raw_output:
                print(data, file=sys.stderr)
                exit(0)
            else:
                console_writer(MaltegoMessage.parse(data))
                exit(0)

        print('ERROR: Received status %d for %s://%s/%s. Are you sure you got the right server?' % (
            r.status,
            'https' if args.ssl else 'http',
            args.host,
            args.transform
        ), file=sys.stderr)
        
        if args.verbose:
            print(r.read(), file=sys.stderr)
    except Exception as e:
        print('ERROR: %s' % e, file=sys.stderr)
    exit(-1)



import getpass
import os

from datetime import date
from importlib import import_module

import sys

import canari
from mrbob.parsing import parse_config

from canari.utils.fs import PushDir

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2016, canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = []


class CanariProject(object):

    marker = '.canari'

    def __init__(self, path=None):
        if not path:
            path = os.getcwd()

        try:
            self._tree = self.project_tree(path)
            self._in_project = True
            self._configuration = parse_config(os.path.join(self.root_dir, '.mrbob.ini'))
            sys.path.insert(0, self.src_dir)
        except ValueError:
            self._tree = dict(
                    root=path,
                    src=path,
                    pkg=path,
                    resources=path,
                    transforms=path
            )
            self._in_project = False
            self._configuration = {'variables': {'created.year': date.today().year, 'author.name': getpass.getuser(),
                                                 'project.name': 'canari', 'canari.version': canari.__version__,
                                                 'author.email': 'transformer@canariframework.com',
                                                 'project.description': '', 'entity.base_name': 'Entity'}}

    @property
    def entities_module(self):
        name = self.name
        return import_module('canari.maltego.entities' if name == 'canari' else '%s.transforms.common.entities' % name)

    @property
    def name(self):
        return self._configuration['variables']['project.name']

    @property
    def root_dir(self):
        return self._tree.get('root')

    @property
    def src_dir(self):
        return self._tree.get('src')

    @property
    def resources_dir(self):
        return self._tree.get('resources')

    @property
    def transforms_dir(self):
        return self._tree.get('transforms')

    @property
    def common_dir(self):
        return self._tree.get('common')

    @property
    def entities_mtz(self):
        return os.path.join(self.resources_dir, 'maltego', 'entities.mtz')

    @property
    def entities_py(self):
        return os.path.join(self.common_dir, 'entities.py')

    @property
    def is_valid(self):
        return self._in_project

    @property
    def configuration(self):
        return self._configuration

    def _project_root(self):
        marker = self.marker
        for i in range(0, 5):
            if os.path.exists(marker) and os.path.isfile(marker):
                return os.path.dirname(os.path.realpath(marker))
            marker = os.path.join('..', marker)
        raise ValueError('Unable to determine project root.')

    def project_tree(self, path):
        with PushDir(path):
            root = self._project_root()

            tree = dict(
                    root=root,
                    src=None,
                    pkg=None,
                    resources=None,
                    transforms=None
            )

            for base, dirs, files in os.walk(root):
                if base.endswith('src'):
                    tree['src'] = base
                elif 'resources' in dirs:
                    tree['pkg'] = base
                elif base.endswith('resources'):
                    tree['resources'] = base
                elif base.endswith('transforms'):
                    tree['transforms'] = base
                    tree['common'] = os.path.join(base, 'common')

            return tree



from past.builtins import basestring

from future.standard_library import install_aliases

install_aliases()

import zlib
import re
from urllib.request import urlopen

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'wordlist'
]


def wordlist(uri, match='(.+?)\n*', ignore='^#.*', strip=None):
    if isinstance(uri, basestring):
        words = []
        if not uri:
            return words
        data = urlopen(uri).read()
        if re.search('\.gz(ip)?$', uri) is not None:
            data = zlib.decompress(data, 16 + zlib.MAX_WBITS)
        if data:
            if callable(match):
                words = match(data)
            else:
                words = re.findall(match, data.decode('utf-8'))
                if ignore is not None:
                    words = [w for w in words if re.search(ignore, w) is None]
                if strip is not None:
                    words = map(lambda x: re.sub(strip, '', x), words)
        return words
    return uri



from __future__ import print_function

import sys

from canari.maltego.utils import highlight
from canari.pkgutils.transform import TransformDistribution
from canari.project import CanariProject
from canari.commands.common import canari_main
from canari.utils.fs import PushDir
from canari.commands.framework import SubCommand, Argument


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.6'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


# Argument parser
@SubCommand(
    canari_main,
    help="Lists transforms in a Canari package.",
    description="Lists transforms in a Canari package."
)
@Argument(
    'package',
    metavar='<package>',
    help='the name of the canari package to list transforms for.'
)
@Argument(
    '-w',
    '--working-dir',
    metavar='[working dir]',
    default=None,
    help="the path that will be used as the working directory for "
         "the canari package being listed (default: ~/.canari/)"
)
def list_transforms(opts):

    try:
        with PushDir(opts.working_dir or CanariProject().src_dir):
            transform_package = TransformDistribution(opts.package)
            for transform_class in transform_package.transforms:
                transform = transform_class()
                print('`- %s: %s' % (highlight(transform.name, 'green', True), transform.description), file=sys.stderr)
                print(highlight('  `- Maltego identifiers:', 'black', True), file=sys.stderr)
                print('    `- %s applies to %s in set %s' % (
                    highlight(transform.name, 'red', False),
                    highlight(transform.input_type._type_, 'red', False),
                    highlight(transform.transform_set, 'red', False)
                ), file=sys.stderr)
                print('', file=sys.stderr)
    except ValueError as e:
        print(e, file=sys.stderr)
        exit(-1)



from __future__ import print_function

import sys

from canari.commands.common import canari_main
from canari.commands.framework import SubCommand

import canari


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help='Show version of Canari framework that is currently active.',
    description='Show version of Canari framework that is currently active.'
)
def version(args):
    print('Canari Framework v%s' % canari.__version__, file=sys.stderr)
    print('Running on Python %s (%s)' % (sys.version, sys.executable))
